#!/usr/bin/env python

# ########################################################################
#
# Copyright (C) 2010 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es, tgabaldon[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################

# This is a wrapper to use when the tree contains information about horizontal 
# gene transfers, it will split the tree in pruned trees by each node marked
# as HGT by the user.

import ete2,random,sys,os
import tree_comparison as tc
import reconciliation_distance as rd
import subtree_reconstruction as sr

def choose_file(item,file_name,num):
	f = file_name
	if item == "TAKE_FILE":
		pass
	else:
		o1 = open("temp_outfile_hgt"+num+".txt","w")
		print >>o1,item
		o1.close()
		f = "temp_outfile_hgt"+num+".txt"
	return f

def compare_trees(trees_hgt1,trees_hgt2,file1,file2,program,out,seed,num_leaves1,num_leaves2,distances,config_file):
	for t1 in trees_hgt1:
		file1 = choose_file(t1,file1,"1")
		l1 = len(ete2.Tree(file1).get_leaves())
		for t2 in trees_hgt2:
			file2 = choose_file(t2,file2,"2")
			l2 = len(ete2.Tree(file2).get_leaves())
			if program == "tc":
				speciation_distance = tc.tree_comparison(file1,file2,config_file,out,seed)
				if "speciation" not in distances:
					distances["speciation"] = []
				weight = float(l1)/float(num_leaves1)*float(l2)/float(num_leaves2)
				distances["speciation"].append(speciation_distance*weight)
			elif program == "rd":
				duplication_distance,all_events_distance = rd.reconciliation_distance(file1,file2,config_file,out,out)
				weight = float(l1)/float(num_leaves1)*float(l2)/float(num_leaves2)
				if duplication_distance != None:
					if "duplications" not in distances:
						distances["duplications"] = []
					distances["duplications"].append(duplication_distance*weight)
				if all_events_distance != None:
					if "all_events" not in distances:
						distances["all_events"] = []
					distances["all_events"].append(all_events_distance*weight)
	return distances

def make_comparisons(trees_hgt1,trees_hgt2,loners_hgt1,loners_hgt2,general_info,num_leaves1,num_leaves2,file1,file2,program,seed,config_file):
	num = 1
	distances = {}
	n = random.randint(0,100000000000)
	out = open("hgt_temporal_"+str(n)+".txt","w")
	if len(trees_hgt1) > 0 and len(trees_hgt2) > 0:
		distances = compare_trees(trees_hgt1,trees_hgt2,file1,file2,program,out,seed,num_leaves1,num_leaves2,distances,config_file)
	else:
		if program == "rd":
			sys.exit("Not enough was left to compare the trees using this method after pruning the HGT nodes.")
	if len(loners_hgt1) > 0 and len(trees_hgt2) > 0:
		if program == "tc":
			deli1 = ""
			deli2 = ""
			for l1 in loners_hgt1:
				for nw2 in trees_hgt2:
					nw2 = choose_file(nw2,file2,"2")
					rf,dist,lenA,lenB,fterm = tc.compare_tree_and_loner(nw2,l1,deli1,deli2,general_info["tree_mode"],general_info["species_position"])
					if "speciation" not in distances:
						distances["speciation"] = []
					weight = float(lenB)/float(num_leaves1)*float(lenA)/float(num_leaves2)
					distances["speciation"].append(fterm*weight)
	if len(loners_hgt2) > 0 and len(trees_hgt1) > 0:
		if program == "tc":
			deli1 = ""
			deli2 = ""
			for l2 in loners_hgt2:
				for nw1 in trees_hgt1:
					nw1 = choose_file(nw1,file1,"1")
					rf,dist,lenA,lenB,fterm = tc.compare_tree_and_loner(nw1,l2,deli1,deli2,general_info["tree_mode"],general_info["species_position"])
					if "speciation" not in distances:
						distances["speciation"] = []
					weight = float(lenA)/float(num_leaves1)*float(lenB)/float(num_leaves2)
					distances["speciation"].append(fterm*weight)
	if len(loners_hgt1) > 0 and len(loners_hgt2) > 0:
		if program == "tc":
			deli1 = ""
			deli2 = ""
			for l1 in loners_hgt1:
				for l2 in loners_hgt2:
					rf,dist,lenA,lenB,fterm = tc.compare_two_loners(l1,l2,deli1,deli2,general_info["tree_mode"],general_info["species_position"])
					if "speciation" not in distances:
						distances["speciation"] = []
					weight = float(lenA)/float(num_leaves1)*float(lenB)/float(num_leaves2)
					distances["speciation"].append(fterm*weight)
	out.close()
	cmd = "rm hgt_temporal_"+str(n)+".txt"
	r = os.popen(cmd)
	return distances

def launch_hgt_run(file1,file2,presence_hgt1,presence_hgt2,config_file,seed,final_outfile,program):
	#Check infiles and obtain information from the configuration file
	nw1,nw2,is_rooted1,is_rooted2,check_species1,check_species2 = tc.check_input_files(file1,file2)
	if program == "tc":
		default,general_info,info_tree1,info_tree2,print_info = tc.parse_configuration_file(config_file,None,is_rooted1,is_rooted2,seed,True)
	elif program == "rd":
		#It is assumed that tree1 is the species tree and tree 2 is the tree that should be reconciled
		default,general_info,info_tree1,info_tree2,print_info = rd.parse_configuration_file(config_file,None,is_rooted1,is_rooted2,True)
		
	#Check the presence of marked horizontal gene transfer events
	if presence_hgt1:
		trees_hgt1,loners_hgt1,num_leaves1 = split_tree_by_hgt(file1)
	else:
		num_leaves1 = len(ete2.Tree(file1).get_leaves())
		trees_hgt1 = ["TAKE_FILE"]
		loners_hgt1 = []
	if presence_hgt2:
		trees_hgt2,loners_hgt2,num_leaves2 = split_tree_by_hgt(file2)
	else:
		num_leaves2 = len(ete2.Tree(file2).get_leaves())
		trees_hgt2 = ["TAKE_FILE"]
		loners_hgt2 = []
	#Make comparisons
	distances = make_comparisons(trees_hgt1,trees_hgt2,loners_hgt1,loners_hgt2,general_info,num_leaves1,num_leaves2,file1,file2,program,seed,config_file)
	if program == "rd":
		d_dup = 0.0
		for d in distances["duplications"]:
			d_dup = d_dup + d
		d_ev = 0.0
		for d in distances["all_events"]:
			d_ev = d_ev + d
		rd.print_results(file1,file2,d_dup,d_ev,final_outfile,print_info)
	elif program == "tc":
		spe_dist = 0.0
		for d in distances["speciation"]:
			spe_dist = spe_dist + d
		print >>final_outfile,"HGT events were detected, only the evolutionary distance has been calculated\n"
		print >>final_outfile,"\tTree 1\tTree 2\tEvolutionary distance"
		print >>final_outfile,"----------------------------------------------------------------------------"
		print >>final_outfile,"Results\t"+file1+"\t"+file2+"\t%.3f" % (spe_dist)
		
	


def split_tree_by_hgt(file_name):
	tree = ete2.PhyloTree(file_name,format=1)
	length_tree = len(tree.get_leaves())
	hgt_nodes = tree.search_nodes(HGT="True")
	raw_trees = []
	for node in hgt_nodes:
		if node.is_root():
			sys.exit("There can not be a HGT event at the root of the tree, please re-root your tree")
		else:
			bush = node.detach()
			raw_trees.append(bush)
	raw_trees.append(tree)
	all_trees = []
	all_loners = []
	for t in raw_trees:
		for node in t.traverse():
			if node.is_leaf():
				pass
			else:
				if len(node.get_children()) < 2:
					node.delete()
		nw = t.write(["evoltype"])
		if len(t.get_leaves()) < 3:
			if t.get_leaf_names()[0] != "NoName":
				all_loners.append(t.get_leaf_names())
		else:
			all_trees.append(nw)
	return all_trees,all_loners,length_tree
	
def check_for_HGT(file_name):
	HGT = False
	try:
		tree = ete2.PhyloTree(file_name,format=1)
		nodes = tree.search_nodes(HGT="True")
		if len(nodes) > 0:
			HGT = True
	except:
		sys.exit("Unable to load Tree, please check your files")
	return HGT