#!/usr/bin/env python

# ########################################################################
#
# Copyright (C) 2010 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################

import ete2
import subtree_reconstruction as ms
import sys
import re
import os
import os.path
from optparse import OptionParser


def load_tables(p,user):
        p._phylomes_table="phylomes_"+user
        p._trees_table="trees_"+user
        p._algs_table="algs_"+user

def connect_to_phylomeDB(info_connect):
  	if not info_connect["host"] and not info_connect["user"] and not info_connect["passwd"] and not info_connect["port"] and not info_connect["db"]:
		p = ete2.PhylomeDBConnector()
	else:
		#p = ete2.PhylomeDBConnector(host="cgenomics",user="web_user",passwd="web.2009",port="3306")
		p = ete2.PhylomeDBConnector(host=info_connect["host"],user=info_connect["user"],passwd=info_connect["passwd"],port=info_connect["port"],db=info_connect["db"])
		if info_connect["user"] != "public" and info_connect["phyuser"]:
			load_tables(p,phyuser)
		elif info_connect["user"] != "public" and not info_connect["phyuser"]:
			sys.exit("Phyome tables need to be loaded when using this user, please provide it with the -U option")
	return p

###########################################################################################################
# This program calculates the bootstrap support for a species tree. For each node in the species tree
# the species grouped together will be searched for in a list of trees (can be a phylome) and the percentage
# of trees supporting each partition is calculated.
###########################################################################################################

###########################################################################################################
# Parser
###########################################################################################################


def load_initial_data(d):
	if d == "g":
		info = {"tree_mode" : "s", "species_position" : "1", "orto_mode" : "s", "species2age" : {}, "species_tree" : "", "root_species" : "", "limit" : 0, "overlap" : False}
	elif d == "u":
		info = {"host": "84.88.66.245","user":"public","passwd":"public","port":3306,"db" : "phylomeDB","phyuser":""}
	else:
		info = {"kind_root" : "m", "spec_filter" : None, "prot_filter" : None, "bootstrap" : None}
	return info
		
def parse_conf_file(file):
	
	#Initialize data:
	connect_info = load_initial_data("u")
	general_info = load_initial_data("g")
	info_tree1 = load_initial_data("t")
	info_tree2 = load_initial_data("t")
	species_limit = "3"
	verbose = False
	for line in open(file):
		line = line.strip()
		if line != "":
			if line[0] != "#":
				if " " in line:
					dad = line.split(" ")
				else:
					dad = line.split("\t")
				if dad[0].lower() == "root_method1":
					kind_root = dad[1].lower()
					if kind_root != "f" and kind_root != "s" and kind_root != "m" and kind_root != "n" and kind_root != "fi" and kind_root != "d":
						kind_root = ""
						sys.exit("The root method provided is not supported, please check the manual for the proper configuration")
					info_tree1["kind_root"] = kind_root
				elif dad[0].lower() == "root_method2":
					kind_root = dad[1].lower()
					if kind_root != "f" and kind_root != "s" and kind_root != "m" and kind_root != "n" and kind_root != "fi" and kind_root != "d":
						kind_root = ""
						sys.exit("The root method provided is not supported, please check the manual for the proper configuration")
					info_tree2["kind_root"] = kind_root
				elif dad[0].lower() == "species_limit":
					species_limit = dad[1].lower()
					if species_limit != "1" and species_limit != "2" and species_limit != "3" and species_limit != "4" and species_limit != "5" and species_limit != "6" and species_limit != "7" and species_limit != "_" and species_limit != "-" and species_limit != "#" and species_limit != ":" and species_limit != "." and species_limit != "all" and species_limit != "phylomedb":
						sys.exit("The species delimiter is not accepted. Please, check the manual for accepted limits for the species name")
				elif dad[0].lower() == "species_position":
					general_info["species_position"] = dad[1].lower()
				elif dad[0].lower() == "root_data":
					if ":" in dad[1]:
						general_info["species2age"][dad[1].split(":")[0]] = int(dad[1].split(":")[1])
					else:
						sys.exit("Information should be provided in the following format:\nspecies_name:level")
				elif dad[0].lower() == "root_species":
					general_info["root_species"] = dad[1]
				elif dad[0].lower() == "orto_mode":
					orto_mode = dad[1].lower()
					if orto_mode != "s" and orto_mode != "r" and orto_mode != "m":
						sys.exit("The orthology detection mode is not valid, please revise the information provided by orto_mode")
					general_info["orto_mode"] = orto_mode

				elif dad[0].lower() == "species_tree":
					general_info["species_tree"] = dad[1]
					
				elif dad[0].lower() == "support1":
					try:
						info_tree1["bootstrap"] = float(dad[1])
					except:
						sys.exit("There was a problem with the support value for tree 1, please check your configuration file")
				elif dad[0].lower() == "support2":
					try:
						info_tree2["bootstrap"] = float(dad[1])
					except:
						sys.exit("There was a problem with the support value for tree 2, please check your configuration file")

				elif dad[0].lower() == "limit":
					try:
						general_info["limit"] = int(dad[1])
					except:
						print "Limit has to have a numerical value, put 0 if you do not whish to use any limit"
						exit()
				elif dad[0].lower() == "verbose":
					verbose = True
				elif dad[0].lower() == "phydbhost":
					connect_info["host"] = dad[1]
				elif dad[0].lower() == "phydbuser":
					connect_info["user"] = dad[1]
				elif dad[0].lower() == "phydbpass":
					connect_info["passwd"] = dad[1]
				elif dad[0].lower() == "phydbport":
					connect_info["port"] = dad[1]
				elif dad[0].lower() == "phydbdatabase":
					connect_info["db"] = dad[1]
				elif dad[0].lower() == "phyuser":
					connect_info["phyuser"] = dad[1]
				else:
					print "WARNING: The information provided in the configuration file is not in available for this program, please check the manual for further information."
					print line
	if general_info["orto_mode"] == "r":
		if general_info["species_tree"] != "":
			if not os.path.exists(general_info["species_tree"]):
				sys.exit("Unable to find the species tree, please check your configuration file")
		else:
			sys.exit("A species tree needs to be provided if reconciliation is chosen as the orthology prediction program, please provide the path to the species tree in the configuration file")
		if info_tree2["bootstrap"] != None or info_tree1["bootstrap"] != None:
			sys.exit("At the moment multifurcated trees cannot be treated with tree reconciliation methods, please switch to species overlap or remove the support limits")
	if general_info["species_position"] != "1" and general_info["species_position"] != "-1":
		sys.exit("The species position has to be either 1 or -1, please check your configuration file")
	if info_tree1["kind_root"] != "n":
		if info_tree1["kind_root"] == "":
			info_tree1["kind_root"] = "m"
		elif info_tree1["kind_root"] == "f" or info_tree1["kind_root"] == "fi":
			if general_info["species2age"] == {}:
				sys.exit("The information for rooting at the farthest node is not provided, please check the manual for further information")
		elif info_tree1["kind_root"] == "s":
			if general_info["root_species"] == "":
				sys.exit("The information needed to root at a given protein is not provided, please check the manual for further information")
				exit()
		elif info_tree1["kind_root"] == "n" and not is_rooted1:
			sys.exit("Tree1 is not rooted, please provide a rooted tree or change your configuration file")
		if info_tree2["kind_root"] != "n":
			if info_tree2["kind_root"] == "":
				info_tree2["kind_root"] = "m"
			elif info_tree2["kind_root"] == "f" or info_tree2["kind_root"] == "fi":
				if general_info["species2age"] == {}:
					sys.exit("The information for rooting at the farthest node is not provided, please check the manual for further information")
			elif info_tree2["kind_root"] == "s":
				if general_info["root_species"] == "":
					sys.exit("The information needed to root at a given protein is not provided, please check the manual for further information")
			elif info_tree2["kind_root"] == "n" and not is_rooted2:
				sys.exit("Tree2 is not rooted, please provide a rooted tree or change your configuration file")
	if general_info["tree_mode"] == "":
		general_info["tree_mode"] = "s"
	if species_limit != "3":
		general_info["tree_mode"] = "s"+species_limit
	return general_info,info_tree1,info_tree2,connect_info,verbose

###########################################################################################################
# Get toplogies from species tree. CAREFUL: Tree must be apropriately rooted!
###########################################################################################################

def obtain_partitions(species_tree):
	#Obtains all the alternative topologies for all the nodes that are candidates to obtain the phylome support
	partitions = []
	for node in species_tree.traverse():
		if node.is_leaf() or node.is_root():
			pass
		else:
			if len(node.get_species()) > 1 and len(node.get_leaves()) == len(species_tree.get_leaves()):
				pass
			else:
				partitions.append(node.get_species())
	return partitions


#file = file name for tree
#kroot = kind of rooting: midpoint(m), spe2age(f), species (s)
#spe2age = dictionary if f rooting is chosen
#tree_mode = trees should be treated as protein trees or species should be identified
#root_species = species where root should be placed if species rooting mode chosen
#check_species = bool, False if there could be repeated name tags at leaves
#limit = integer, indicates whether a limit for the number of subtrees should be enabled

#Obtains a list of subtrees for each tree to be considered, only subtrees that contain the seed will be used
def get_subtrees(file_name,general_info,info_tree2,check_species,only_orthologs,seed,list_subtrees):
	colection,total_tree_length,deli,stop = ms.make_subtrees(file_name,general_info,info_tree2,check_species)
	col = []
	for tree in colection:
		if type(tree) == str:
			col.append(tree)
	collection = []
	if only_orthologs and not seed == None:
		for nw in col:
			try:
				tree = ete2.PhyloTree(nw)
				if seed in tree.get_leaf_names():
					collection.append(nw)
			except:
				pass
	else:
		for nw in col:
			try:
				tree = ete2.PhyloTree(nw)
				collection.append(nw)
			except:
				pass
	if seed:
		list_subtrees[seed] = collection
	else:
		code = "No_seed_"+str(len(list_subtrees))
		list_subtrees[code] = collection
	return list_subtrees

###########################################################################################################
# Read inputs
###########################################################################################################


def get_list_subtrees(subtree_file,only_orthologs,verbose,info_tree2,general_info):
	list_subtrees = {}
	check_species = True
	if verbose:
		total = 0
		for line in open(subtree_file):
			total = total+1
		i = 0
	for line in open(subtree_file):
		line = line.strip()
		if " " in line:
			files = line.split(" ")
		else:
			files = line.split("\t")
		if len(files) == 2:
			if files[0] in files[1]:
				seed = files[0]
			else:
				seed = None
			if "(" in files[1] and ")" in files[1] and ";" in files[1]:
				file = open("tmp_tree.txt","w")
				print >>file,files[1]
				file.close()
				file_name = "tmp_tree.txt"
			else:
				file_name = files[1]
		else:
			if "(" in line and ")" in line and ";" in line:
				file = open("tmp_tree.txt","w")
				print >>file, line
				file.close()
				file_name = "tmp_tree.txt"
			else:
				file_name = line
			seed = None
		list_subtrees = get_subtrees(file_name,general_info,info_tree2,check_species,only_orthologs,seed,list_subtrees)
		if verbose:
			i = i + 1
			if i%100 == 0:
				print str(i)+" out of "+str(total)
	if os.path.exists("tmp_tree.txt"):
		cmd = "rm tmp_tree.txt"
		resu = os.popen(cmd)
	return list_subtrees


def read_phylome(phyID,info_connect,only_orthologs,verbose,info_tree2,general_info):
	p = connect_to_phylomeDB(info_connect)
	list_subtrees = {}
	if verbose:
		total = len(p.get_seed_ids(phyID))
		i = 0
	for seed in p.get_seed_ids(phyID):
		check_species = True
		if p.get_best_tree(seed,phyID)[2] != None:
			file = open("tmp_tree.txt","w")
			print >>file,p.get_best_tree(seed,phyID)[2].write()
			file.close()
			list_subtrees = get_subtrees("tmp_tree.txt",general_info,info_tree2,check_species,only_orthologs,seed,list_subtrees)
		if verbose:
			i = i + 1
			if i%100 == 0:
				print str(i)+" out of "+str(total)
	if os.path.exists("tmp_tree.txt"):
		cmd = "rm tmp_tree.txt"
		resu = os.popen(cmd)
	return list_subtrees

def locate_node(list_spe,tree):
	species = list_spe.split("-")
	spe1 = tree.get_leaves_by_name(species[0])[0]
	spe2 = tree.get_leaves_by_name(species[1])[0]
	common = spe1.get_common_ancestor(spe2)
	while len(common.get_leaves()) != len(species) and not common.is_root():
		common = common.up
	return common



###########################################################################################################
# Create outfiles
###########################################################################################################


# Looks whether two species apear in sister nodes in a given tree
def is_sister(tree,spe1,spe2):
	node = tree.get_leaves_by_name(spe1)[0]
	sis = node.get_sisters()[0]
	if spe2 in sis.get_leaf_names():
		s = True
	else:
		s = False
	return s

# Prints the species tree with the node support, branch distances are ommited
def assign_support_to_tree(total,support,tree,outtree,outnw):
	for node in tree.traverse():
		node.dist = 1.0
	for node in tree.traverse():
		if node.is_leaf():
			node.support = 1.0
		else:
			for p in total:
				species = set(p.split("-"))
				intersection = node.get_species().intersection(species)
				if len(intersection) == len(node.get_species())and len(intersection) == len(species):
					node.support = support[p] / total[p]
	IP = ete2.TreeImageProperties()
	IP.draw_branch_length = True
	IP.force_topology = True
	tree.write(outfile=outnw)
	tree.render(outtree,img_properties=IP)
	
	
def bootstrap_support(species_tree_file,tree_file,phyID,conf_file,outtree,outnw):
	#Parse options
	if conf_file:
		general_info, info_tree1, info_tree2, connect_info, verbose = parse_conf_file(conf_file)
	else:
		connect_info = load_initial_data("u")
		general_info = load_initial_data("g")
		info_tree1 = load_initial_data("t")
		info_tree2 = load_initial_data("t")
		species_limit = "3"
		verbose = True
	
	#SPECIES TREE INFILE
	if not os.path.exists(species_tree_file):
		sys.exit("Unable to find the species tree file, please, check your command line")
	else:
		t = ete2.PhyloTree(species_tree_file)
		if len(t.get_children()) == 3:
			sys.exit("Species tree needs to be rooted, else unexpected partitions may be computed")
		else:
			partitions = obtain_partitions(ete2.PhyloTree(species_tree_file))
	
	#LISTS OF TREES
	if verbose:
		print "Getting pruned trees"

	only_orthologs = True

	if tree_file:
		if not os.path.exists(tree_file):
			sys.exit("The file containing the list of trees was not found, please check your command line")
		else:
			list_subtrees = get_list_subtrees(tree_file,only_orthologs,verbose,info_tree2,general_info)
	elif phyID:
		list_subtrees = read_phylome(phyID,connect_info,only_orthologs,verbose,info_tree2,general_info)
	if len(list_subtrees) == 0:
		sys.exit("No trees were found, please check your input data. If a phylomeID was provided, check whether this is available to the public user")
	if verbose:
		print "Getting support"

	total = {}
	support = {}
	for sp in partitions:
		for code,trees in list_subtrees.iteritems():
			t = 0.0
			s = 0.0
			for tree in trees:
				tree = ete2.PhyloTree(tree)
				partitions_tree = obtain_partitions(tree)
				intersection = tree.get_species().intersection(sp)
				if len(list(intersection)) == len(list(sp)):
					if len(list(intersection)) != len(list(tree.get_species())):
						t = t + 1.0
						taken = verify_partition(tree,partitions_tree,sp)
						if taken:
							s = s + 1.0
			if t > 0.0:
				string = "-".join(sp)
				if string not in total:
					total[string] = 0.0
				total[string] = total[string] + 1.0
				if string not in support:
					support[string] = 0.0
				support[string] = support[string] + s/t
	#for string in support:
		#print string,support[string],total[string]
	tree = assign_support_to_tree(total,support,ete2.PhyloTree(species_tree_file),outtree,outnw)

def verify_partition(tree,partitions,p):
	perfect_match = ""
	smallest = ""
	for species_p in partitions:
		intersection = species_p.intersection(p)
		if len(list(intersection)) == len(list(species_p)) and len(list(intersection)) == len(list(p)):
			perfect_match = species_p
		elif p.issubset(species_p) and (smallest == "" or len(list(species_p)) < len(list(smallest))):
			smallest = species_p
	taken = ""
	if not perfect_match:
		take = True
		for spe in smallest:
			if spe not in p and spe in tree.get_species():
				take = False
		if take:
			taken = smallest
	else:
		taken = perfect_match
	return taken
