#!/usr/bin/env python

# ########################################################################
#
# Copyright (C) 2010 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################

import ete2,sys,os
import subtree_reconstruction as sr
import tree_comparison as tc

def load_initial_data(d):
	if d == "g":
		info = {"tree_mode" : "s", "species_position" : "1", "species2age" : {}, "root_species" : "","orto_mode":"s","species_tree":""}
	elif d == "p":
		info = {"pall" : False, "pdupli" : False, "peve" : False, "pconfig" : False, "verbose" : False, "oneline" : False}
	else:
		info = {"kind_root" : "m"}
	return info

def parse_configuration_file(options_conf_file,outfile,is_rooted1,is_rooted2,pconfig):
	if not options_conf_file or not os.path.exists(options_conf_file):
		
		#Load initial parameters
		print_info = load_initial_data("p")
		general_info = load_initial_data("g")
		info_stree = load_initial_data("t")
		info_tree = load_initial_data("t")
		default = True
		#Modify needed info:
		print_info["pall"] = True
		print_info["verbose"] = True
		if is_rooted1:
			info_stree["kind_root"] = "n"
		if is_rooted2:
			info_tree["kind_root"] = "n"
		if pconfig and not print_info["oneline"]:
			print_configuration(default,general_info,info_stree,info_tree,print_info,outfile)
	else:
		general_info,info_stree,info_tree,print_info = parse_conf_file(options_conf_file,is_rooted1,is_rooted2)
		default = False
		if pconfig and not print_info["oneline"]:
			print_configuration(default,general_info,info_stree,info_tree,print_info,outfile)
	return default,general_info,info_stree,info_tree,print_info

def parse_conf_file(file,is_rooted1,is_rooted2):
	
	#Initialize data:
	print_info = load_initial_data("p")
	general_info = load_initial_data("g")
	info_stree = load_initial_data("t")
	info_tree = load_initial_data("t")
	species_limit = "3"
	
	#Read configuration file
	for line in open(file):
		line = line.strip()
		if line != "":
			if line[0] != "#":
				if " " in line:
					dad = line.split(" ")
				else:
					dad = line.split("\t")
				if dad[0].lower() == "tree_mode":
					general_info["tree_mode"] = dad[1].lower()
				elif dad[0].lower() == "species_position":
					general_info["species_position"] = dad[1].lower()
				elif dad[0].lower() == "root_method1":
					kind_root1 = dad[1].lower()
					if kind_root1 != "f" and kind_root1 != "s" and kind_root1 != "m" and kind_root1 != "n" and kind_root1 != "d" and kind_root1 != "fi":
						sys.exit("The root method provided is not supported, please check the manual for the proper configuration")
					if is_rooted1:
						info_stree["kind_root"] = "n"
					else:
						info_stree["kind_root"] = kind_root1
				elif dad[0].lower() == "root_method2":
					kind_root2 = dad[1].lower()
					if kind_root2 != "f" and kind_root2 != "s" and kind_root2 != "m" and kind_root2 != "n" and kind_root2 != "d" and kind_root2 != "fi":
						sys.exit("The root method provided is not supported, please check the manual for the proper configuration")
					if is_rooted2:
						info_tree["kind_root"] = "n"
					else:
						info_tree["kind_root"] = kind_root2
				elif dad[0].lower() == "species_limit":
					species_limit = dad[1].lower()
					if species_limit != "1" and species_limit != "2" and species_limit != "3" and species_limit != "4" and species_limit != "5" and species_limit != "6" and species_limit != "7" and species_limit != "_" and species_limit != "-" and species_limit != "#" and species_limit != ":" and species_limit != "all" and species_limit != "phylomedb":
						print "The species delimiter is not accepted. Please, check the manual for accepted limits for the species name"
						exit()
				elif dad[0].lower() == "root_data":
					if ":" in dad[1]:
						general_info["species2age"][dad[1].split(":")[0]] = int(dad[1].split(":")[1])
					else:
						sys.exit("Information should be provided in the following format:\nspecies_name:level")
				elif dad[0].lower() == "root_species":
					general_info["root_species"] = dad[1]
				elif dad[0].lower() == "print_all":
					print_info["pall"] = True
				elif dad[0].lower() == "print_oneline":
					print_info["oneline"] = True
				elif dad[0].lower() == "print_all_events_distance":
					print_info["peve"] = True
				elif dad[0].lower() == "print_duplications_distance":
					print_info["pdupli"] = True 
				elif dad[0].lower() == "print_configuration":
					print_info["pconfig"] = True
				elif dad[0].lower() == "verbose":
					print_info["verbose"] = True
				else:
					print "WARNING: The information provided in the configuration file is not in a recognised format or does not belong to this program, please check the manual for further information."
					print line
	#Checking necessary info
	if info_stree["kind_root"] != "n":
		if info_stree["kind_root"] == "":
			info_stree["kind_root"] = "m"
		elif info_stree["kind_root"] == "f" or info_stree["kind_root"] == "fi":
			if general_info["species2age"] == {}:
				sys.exit("The information for rooting at the farthest node is not provided, please check the manual for further information")
		elif info_stree["kind_root"] == "s":
			if general_info["root_species"] == "":
				sys.exit("The information needed to root at a given protein is not provided, please check the manual for further information")
		elif info_stree["kind_root"] == "n" and not is_rooted1:
			sys.exit("Tree1 is not rooted, please provide a rooted tree or change your configuration file")

	if info_tree["kind_root"] != "n":
		if info_tree["kind_root"] == "":
			info_tree["kind_root"] = "m"
		elif info_tree["kind_root"] == "f" or info_tree["kind_root"] == "fi":
			if general_info["species2age"] == {}:
				sys.exit("The information for rooting at the farthest node is not provided, please check the manual for further information")
		elif info_tree["kind_root"] == "s":
			if general_info["root_species"] == "":
				sys.exit("The information needed to root at a given protein is not provided, please check the manual for further information")
		elif info_tree["kind_root"] == "n" and not is_rooted2:
			sys.exit("Tree2 is not rooted, please provide a rooted tree or change your configuration file")
	if general_info["tree_mode"] == "":
		general_info["tree_mode"] = "s"
	if species_limit != "3":
		general_info["tree_mode"] = "s"+species_limit
	if general_info["species_position"] != "1" and general_info["species_position"] != "-1":
		sys.exit("The species position has to be either 1 or -1, please check your configuration file")
	if not print_info["pdupli"] and not print_info["peve"] and not print_info["pall"] and not print_info["oneline"]:
		print_info["pall"] = True
		
	if print_info["pall"]:
		print_info["pconfig"] = True
		print_info["pdupli"] = False
		print_info["peve"] = False
		print_info["verbose"] = True
		print_info["oneline"] = False
	return general_info,info_stree,info_tree,print_info

def reconcile_tree(tree_file,species_tree,general_info,info_tree):
	try:
		tree = sr.set_chosen_outgroup(tree_file,info_tree["kind_root"],general_info["species2age"],general_info["tree_mode"],general_info["species_position"],general_info["root_species"],general_info["orto_mode"],general_info["species_tree"])
		rtree, events = tree.reconcile(species_tree)
	except:
		sys.exit("Unable to reconcile the tree")
	return rtree

def calculate_loss(rtree):
	i = 0
	for node in rtree.iter_descendants():
		code = "n"+str(i)
		i = i + 1
		node.add_features(nodename=code)
	llista_perdues=[]
	llista_perdues_total=[]
	for node in rtree.search_nodes(evoltype="L"):
		agafar = True
		for leaf in node.get_leaves():
			if len(leaf.name) > 3:
				agafar = False
		if agafar:
			parent = node.up
			if parent.nodename not in llista_perdues_total:
				llista_perdues.append(node.nodename)
				llista_perdues_total.append(node.nodename)
			else:
				llista_perdues_total.append(node.nodename)
	return llista_perdues

def get_species_tree(stree_file,general_info,info_stree):
	if not os.path.exists(stree_file):
		sys.exit("Unable to find the species tree file, please check the location of your file")
	species_tree = sr.set_chosen_outgroup(stree_file,info_stree["kind_root"],general_info["species2age"],general_info["tree_mode"],general_info["species_position"],general_info["root_species"],general_info["orto_mode"],general_info["species_tree"])
	return species_tree


def get_codes(filename):
	codis = []
	for line in open(filename):
		line = line.strip()
		code = line.split(".")[0]
		codis.append(code)
	return codis

def print_results(stree_file,tree_file,dupli,events,outfile,print_info):
	if print_info["verbose"]:
		print >>outfile,"\n#########################################################"
		print >>outfile,"# Reconciliation distance between:"
		print >>outfile,"# Species tree: "+stree_file
		print >>outfile,"# Tree: "+tree_file
		print >>outfile,"#########################################################"
		print >>outfile,"\n"
		if print_info["pall"] or print_info["pdupli"]:
			print >>outfile,"Number of duplications: "+str(dupli)
		if print_info["pall"] or print_info["peve"]:
			print >>outfile,"Total number of events (duplication + loss): "+str(events)
		print >>outfile,"\n"
	else:
		if print_info["pall"] or print_info["pdupli"]:
			print >>outfile,str(dupli)
		if print_info["pall"] or print_info["peve"]:
			print >>outfile,str(events)

def print_oneline(stree_file,tree_file,dupli,events,outfile,print_info):
	print >>outfile,"\tTree 1\tTree 2\tDuplications\tAll events (duplications + loss)"
	print >>outfile,"----------------------------------------------------------------------------"
	print >>outfile,"Results\t"+stree_file+"\t"+tree_file+"\t"+str(dupli)+"\t"+str(events)

def print_configuration(default,general_info,info_stree,info_tree,print_info,outfile):
	if default:
		print >>outfile,"No valid configuration file provided. Running on default values:"
	else:
		print >>outfile,"This is your current configuration:"
		
	print >>outfile,"\n**Information for Species tree**\n"
	if info_stree["kind_root"] == "m":
		print >>outfile,"Root = Midpoint"
	elif info_stree["kind_root"] == "f":
		print >>outfile,"Root = Farthest species"
	elif info_stree["kind_root"] == "fi":
		print >>outfile,"Root = Farthest species improved"
	elif info_stree["kind_root"] == "s":
		print >>outfile,"Root = Specific protein or species"
	elif info_stree["kind_root"] == "n":
		print >>outfile,"Root = Already rooted"
	elif info_stree["kind_root"] == "d":
		print >>outfile,"Root = Minimize duplication nodes"
		
	print >>outfile,"\n**Information for Tree**\n"
	if info_tree["kind_root"] == "m":
		print >>outfile,"Root = Midpoint"
	elif info_tree["kind_root"] == "f":
		print >>outfile,"Root = Farthest species"
	elif info_tree["kind_root"] == "fi":
		print >>outfile,"Root = Farthest species improved"
	elif info_tree["kind_root"] == "s":
		print >>outfile,"Root = Specific protein or species"
	elif info_tree["kind_root"] == "n":
		print >>outfile,"Root = Already rooted"
	elif info_tree["kind_root"] == "d":
		print >>outfile,"Root = Minimize duplication nodes"	
	
	print >>outfile, "\n**Printing information**\n"
	if print_info["pall"]:
		print >>outfile,"Print all = Yes"
	else:
		if print_info["pdupli"]:
			print >>outfile,"Print number of duplications = Yes"
		else:
			print >>outfile,"Print number of duplications = No"
		if print_info["peve"]:
			print >>outfile,"Print total number of events = Yes"
		else:
			print >>outfile,"Print total number of events = No"
		if print_info["verbose"]:
			print >>outfile,"Print verbose = Yes"
		else:
			print >>outfile,"Print verbose = No"

def reconciliation_distance(stree_file,tree_file,conf_file,output_name,outfile):
	nw1,nw2,is_rooted1,is_rooted2,check_species1,check_species2 = tc.check_input_files(stree_file,tree_file)
	if not check_species1:
		sys.exit("The species tree cannot contain duplications")
	default,general_info,info_stree,info_tree,print_info = parse_configuration_file(conf_file,outfile,is_rooted1,is_rooted2,True)
	species_tree = get_species_tree(stree_file,general_info,info_stree)
	rtree = reconcile_tree(tree_file,species_tree,general_info,info_tree)
	if rtree != "":
		loss = calculate_loss(rtree)
		dupli = rtree.search_nodes(evoltype="D")
		events = len(loss) + len(dupli)
		if print_info["oneline"]:
			print_oneline(stree_file,tree_file,len(dupli),events,outfile,print_info)
		else:
			print_results(stree_file,tree_file,len(dupli),events,outfile,print_info)
	return len(dupli),events



