#!/usr/bin/env python

# ########################################################################
#
# Copyright (C) 2010 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################

import sys,ete2
from random import randint
import subtree_reconstruction as sr

########################################################################################################################
# Ensures that the tree we are adding is not yet present in the list of new trees
########################################################################################################################

def check_presence(list_trees,tree1):
	trobat = False
	for t in list_trees:
		if not trobat:
			tree2 = sr.load_tree(t,tree_mode,species_position,0)
			rf = calc_rf(tree1,tree2)
			if rf == 0:
				trobat = True
	return trobat

def calc_rf(tree1,tree2):
	r1 = get_partitions(tree1)
	r2 = get_partitions(tree2)
	inters = r1.intersection(r2)
	if len(r1) == len(r2):
		rf = (len(r1) - len(inters)) * 2
	else :
		rf = (len(r1) - len(inters)) + (len(r2) - len(inters))
	return rf

def get_partitions(tree):
	orig_proteins = set(tree.get_leaf_names())
	left = []
	right = []
	joint = []
	for node in tree.get_descendants():
		if not node.is_leaf():
			list1 = set(node.get_leaf_names())
			if node.get_sisters():
				ls1 = sorted(list(list1))
				r = '|'.join(ls1)
				right.append(r)
	right = set(right)
	return right

########################################################################################################################
# STRICT METHOD
########################################################################################################################

def get_species_from_partition(partitions,ch,species_children,key):
	species = []
	for l in ch.get_leaves():
		if l.name not in partitions:
			if l.species not in species:
				species.append(l.species)
		else:
			for s in species_children[l.name]:
				if s not in species:
					species.append(s)
	species = set(species)
	return species

#We reconstruct all possible trees containing multifurcations when dealing with speciation events and assuming the least number of duplications
def reconstruct_partitions(partitions,species_children):
	ntrees = {}
	#print_key = "x0"
	#For each multifurcated node
	for key,child in partitions.iteritems():
		ntrees[key] = []
		duplicates = []
		speciates = []
		children = child.get_children()
		#print key, [ch.get_species() for ch in children]
		#Split children between those that are part of a duplication and those that are not
		d = 0
		#For each child we check whether it has a duplicate in the node
		for ch1 in children:
			s = True
			for ch2 in children:
				if ch1 != ch2:
					species1 = get_species_from_partition(partitions,ch1,species_children,key)
					species2 = get_species_from_partition(partitions,ch2,species_children,key)
					overlap = species1.intersection(species2)
					if list(overlap) != []:
						s = False
			if s:
				speciates.append(ch1)
			else:
				duplicates.append(ch1)
		#if key == print_key:
			#child.show()
			#for d in duplicates:
				#d.show()
		pars = {}
		#For each pair for nodes in the duplicate folder we look wether they are linked (they are paralogs) or they are unlinked (orthologs)
		rename = {}
		d = 0
		for ch in duplicates:
			rename["d"+str(d)] = ch
			rename[ch] = "d"+str(d)
			#if key == print_key:
				#print "RENAMING"
				#print "d"+str(d),ch.get_leaf_names()
			d = d + 1
		for ch1 in duplicates:
			pars[rename[ch1]] = {}
			for ch2 in duplicates:
				if ch1 != ch2:
					species1 = get_species_from_partition(partitions,ch1,species_children,key)
					species2 = get_species_from_partition(partitions,ch2,species_children,key)
					inters = species1.intersection(species2)
					if inters != set([]):
						pars[rename[ch1]][rename[ch2]] = "linked"
					else:
						pars[rename[ch1]][rename[ch2]] = "unlinked"
		#if key == print_key:
			#print "PARS"
			#print pars
		#MAKE GROUPS OF UNLINKED CHILDREN
		i = 0
		groups = {}
		for ch in pars:
			groups[i] = []
			groups[i].append(ch)
			i = i + 1

		cont = True
		while cont:
			cont = False
			for ch in pars:
				gr = 0
				maxim = len(groups)
				while gr < maxim:
					trobat = True
					for member in groups[gr]:
						if member != ch:
							if pars[member][ch] == "linked":
								trobat = False
						else:
							trobat = False
					if trobat:
						new_group = []
						for members in groups[gr]:
							new_group.append(members)
						new_group.append(ch)
						afegir = True
						for g in groups:
							trobat = True
							for m in new_group:
								if m not in groups[g]:
									trobat = False
							if trobat:
								afegir = False
						if afegir:
							groups[i] = new_group
							i = i + 1
							cont = True
					gr = gr + 1
		unique_groups = []
		for gr1 in groups:
			subgrup = False
			for gr2 in groups:
				if set(groups[gr1]).issubset(set(groups[gr2])) and gr1 != gr2: 
					subgrup = True
			if not subgrup:
				unique_groups.append(gr1)
		#if key == print_key:
			#for ug in unique_groups:
				#print ug,groups[ug]


		#We construct the subtrees
		for g in unique_groups:
			t = ete2.PhyloTree()
			for s in speciates:
				t.add_child(s)
			for gr in groups[g]:
				t.add_child(rename[gr])
			ntrees[key].append(t.write())
	return ntrees



# Substitutes the X annotations by the actual partitions. When X partitions are dependant
# it makes all the possible combinations

def completely_resolve_trees(tree,new_trees,tree_mode,species_position):
	if type(tree) != str:
		tree = tree.write()
	allc = [tree]
	all_resolved_combinations = []
	fully_resolved = False
	while not fully_resolved:
		complete_collection = []
		for k in new_trees:
			for t in allc:
				if k in sr.load_tree(t,tree_mode,species_position,0).get_leaf_names():
					for trees in new_trees[k]:
						tree = sr.load_tree(t,tree_mode,species_position,0)
						node = tree.search_nodes(name=k)
						if len(node) == 1:
							node = node[0]
							parent = node.up
							node.detach()
							parent.add_child(sr.load_tree(trees,tree_mode,species_position,0))
							for n in tree.traverse():
								if not n.is_leaf():
									if len(n.get_children()) == 1:
										n.delete()
						complete_collection.append(tree.write())
		allc = []
		for t in complete_collection:
			trobat = False
			for k in new_trees:
				tr = sr.load_tree(t,tree_mode,species_position,0)
				if k in tr.get_leaf_names():
					trobat = True
			if trobat:
				if t not in allc:
					allc.append(t)
			else:
				if t not in all_resolved_combinations:
					all_resolved_combinations.append(t)
		if allc == []:
			fully_resolved = True
	return all_resolved_combinations
	
########################################################################################################################
# Basics
########################################################################################################################

def collapse_nodes_low_bootstrap(tree, min_support):
	for n in tree.traverse():
		if not n.is_leaf():
			if n.support < min_support:
				n.delete()
	return tree

########################################################################################################################
# DELETES INTRASPECIFIC DUPLICATIONS OF NODES D AND NODES MD
########################################################################################################################

def delete_intraspecific(tree):
	list_deleted = {}
	leaves_to_delete = []
	intrasp_seq = ""
	associated_list = {}
	for node in tree.traverse("postorder"):
		if not node.is_leaf():
			leaves = []
			if node.evoltype == "D":
				if len(node.get_species()) == 1:
					leaves = node.get_leaf_names()
					associated_list,leaves_to_delete = reformate_intrasp_list(leaves,associated_list,leaves_to_delete)
	leaves = set(tree.get_leaf_names())
	leaves_to_delete = set(leaves_to_delete)
	keepers = leaves.difference(leaves_to_delete)
	tree.prune(keepers)
	leaves_to_delete = []
	for node in tree.traverse("postorder"):
		if not node.is_leaf():
			leaves = []
			if node.evoltype == "MD":
				unique = {}
				for child in node.get_children():
					if len(child.get_leaves()) == 1:
						leaf = list(child.get_species())[0]
						if leaf not in unique:
							unique[leaf] = 0
						unique[leaf] = unique[leaf] + 1
				for leaf,val in unique.iteritems():
					if val > 1:
						nodes = node.search_nodes(species = leaf)
						leaves = [leaf.get_leaf_names()[0] for leaf in nodes]
						associated_list,leaves_to_delete = reformate_intrasp_list(leaves,associated_list,leaves_to_delete)

	leaves = set(tree.get_leaf_names())
	leaves_to_delete = set(leaves_to_delete)
	keepers = leaves.difference(leaves_to_delete)
	tree.prune(keepers)
	tree = annotate_tree(tree)
	return tree,associated_list

def reformate_intrasp_list(leaves,associated_list,leaves_to_delete):
	trobat = []
	intrasp_seq = ""
	if intrasp_seq == "" or intrasp_seq not in leaves:
		intrasp_seq = leaves[0]
		associated_list[leaves[0]] = []
	for l in leaves:
		if l != intrasp_seq:
			if l not in leaves_to_delete:
				leaves_to_delete.append(l)
			if l in associated_list:
				for ld in associated_list[l]:
					if ld not in associated_list[intrasp_seq]:
						associated_list[intrasp_seq].append(ld)
				del associated_list[l]
			if l not in associated_list[intrasp_seq]:
				associated_list[intrasp_seq].append(l)
	return associated_list,leaves_to_delete

########################################################################################################################
# ANNOTATES PROTEIN NODES WITH THEIR EVOLUTIVE TYPE
########################################################################################################################

def get_evol_events(tree,orto_mode):
	if orto_mode == "s" or orto_mode == "r":
		tree = annotate_tree(tree)
	elif orto_mode == "m":
		tree = check_tree_annotations(tree)
	return tree

def check_tree_annotations(tree):
	for node in tree.traverse():
		if node.is_root():
			node.evoltype="R"
		elif not node.is_leaf():
			if len(node.get_children) > 2:
				if node.evoltype == "D" or node.evoltype =="S":
					node.evoltype = "M"+node.evoltype
	return tree

def annotate_tree(tree):
	for node in tree.traverse("postorder"):
	  if len(node.get_children()) != 2:
		  if node.is_root():
			  node.add_feature("evoltype","R")
		  else:
			  dupli = False
			  for ch1 in node.get_children():
				  for ch2 in node.get_children():
					  if ch1 != ch2:
						  spe1 = ch1.get_species()
						  spe2 = ch2.get_species()
						  common = spe1.intersection(spe2)
						  if common != set([]):
							  dupli = True
			  if dupli:
				  node.add_feature("evoltype","MD")
			  else:
				  node.add_feature("evoltype","MS")
	  else:
		  common = []
		  spe1 = node.get_children()[0].get_species()
		  spe2 = node.get_children()[1].get_species()
		  common = spe1.intersection(spe2)
		  if common != set([]):
			  node.add_feature("evoltype","D")
		  else:
			  node.add_feature("evoltype","S")
	return tree

	
###########################################################################################
# Main program, needs to get a rooted tree
###########################################################################################

def generate_resolved_trees(tree,orto_mode,tree_mode,species_position):
	if type(tree) == str:
		tree = sr.load_tree(tree,tree_mode,species_position,0)
	#Annotate nodes
	tree = get_evol_events(tree,orto_mode)
	#Remove internal duplicates, if possible
	tree,intraspecific_deleted = delete_intraspecific(tree)
	#get multifurcated nodes
	x = 0
	multi_children = {}
	species_children = {}
	multi_nodes = tree.search_nodes(evoltype="MD")
	if multi_nodes != []:
		for node in multi_nodes:
			parent = node.up
			children = node.detach()
			parent.add_child(name="x"+str(x))
			multi_children["x"+str(x)] = children
			species_children["x"+str(x)] = children.get_species()
			x=x+1
		nw = tree.write()
		#tree.show()
		new_trees = reconstruct_partitions(multi_children,species_children)
		resolved_trees = completely_resolve_trees(tree,new_trees,tree_mode,species_position)
	else:
		resolved_trees = [tree.write()]
	return resolved_trees,intraspecific_deleted
