#!/usr/bin/env python

# ########################################################################
#
# Copyright (C) 2010 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################


import ete2
import solve_multifurcations as sm
import sys

class Manual_Events():
	def __init__(self):
		self.etype         = None   # 'speciation' or 'duplication'
		self.in_seqs       = []
		self.out_seqs      = []

########################################################################################################
# Make subtrees
########################################################################################################

def make_subtrees(file,general_info,info_tree,check_species):
	#Subtrees are generated from a tree which is passed through a file
	induplis=[]
	outduplis=[]
	llargada=[]
	ortholog=[]
	all_trees = []
	loners=[]
	tree = load_tree(file,general_info["tree_mode"],general_info["species_position"],0)
	if info_tree["prot_filter"] != None:
		s = len(tree.get_leaves_by_name(info_tree["prot_filter"]))
		if s == 0:
			info_tree["prot_filter"] = None
	stop = False
	#Check whether the tree has more than one species:
	if len(tree.get_species()) == 1:
		stop = "ONESPECIES"
	if stop != "ONESPECIES":
		if not check_species:
			tree = rename_leaves(tree)
		if info_tree["bootstrap"] != None:
			tree = sm.collapse_nodes_low_bootstrap(tree,info_tree["bootstrap"])
		tree = set_chosen_outgroup(tree,info_tree["kind_root"],general_info["species2age"],general_info["tree_mode"],general_info["species_position"],general_info["root_species"],general_info["orto_mode"],general_info["species_tree"])
		if tree == "NOTROOTED":
			list_trees,total_length,deleted,stop = "",0,"","NOTROOTED"
		else:
			total_length=len(tree.get_leaf_names())
			resolved_trees,deleted = sm.generate_resolved_trees(tree.write(),general_info["orto_mode"],general_info["tree_mode"],general_info["species_position"])
			for tree in resolved_trees:
				list_trees=[]
				tree = load_tree(tree,general_info["tree_mode"],general_info["species_position"],0)
				if not len(tree.get_leaf_names())<2:
					tree = check_root(tree)
					nw=tree.write()
					induplis,outduplis,llargada = get_duplications(tree,general_info["orto_mode"],general_info["species_tree"])
					if induplis == []:
						list_trees.append(nw)
					else:
						ortholog,stop=generate_orthology_lists(induplis,outduplis,tree,llargada,general_info["tree_mode"],general_info["species_position"],general_info["limit"],general_info["orto_mode"])
						if not stop:
							list_trees=get_trees(ortholog,nw,general_info["tree_mode"],general_info["species_position"])
				for t in list_trees:
					all_trees.append(t)
			check_subtrees_for_duplications(all_trees,general_info["tree_mode"],general_info["species_position"],general_info["orto_mode"])
	else:
		list_trees,total_length,deleted = "",0,""
	colection = filter_colection(all_trees,general_info,info_tree)
	return colection,total_length,deleted,stop

def filter_colection(colection,general_info,info_tree):
	if info_tree["prot_filter"] != None:
		useful = []
		for c in colection:
			if type(c) == str:
				tree = load_tree(c,general_info["tree_mode"],general_info["species_position"],0)
				if info_tree["prot_filter"] in tree.get_leaf_names():
					useful.append(c)
			elif type(c) == list:
				if info_tree["prot_filter"] in c:
					useful.append(c)
	elif info_tree["spec_filter"] != None:
		useful = []
		for c in colection:
			if type(c) == str:
				tree = load_tree(c,general_info["tree_mode"],general_info["species_position"],0)
				if info_tree["spec_filter"] in tree.get_species():
					useful.append(c)
			elif type(c) == list:
				if general_info["tree_mode"] == "s":
					if general_info["species_position"] == "1":
						species = [s[0:3] for s in c]
					elif general_info["species_position"] == "-1":
						species = [s[len(s)-3:] for s in c]
				else:
					num = general_info["tree_mode"].replace("s","")
					if num == "#" or num == ":" or num == "_" or num == "-" or num == ".":
						if general_info["species_position"] == "1":
							species = [s.split(num)[0] for s in c]
						elif general_info["species_position"] == "-1":
							species = [s.split(num)[-1] for s in c]
					elif num == "all":
						species = [s for s in c]
					elif num == "phylomedb":
						species = [s.split("_")[1] for s in c]
					else:
						num = int(num)
						if general_info["species_position"] == "1":
							species = [s[0:num] for s in c]
						elif general_info["species_position"] == "-1":
							species = [s[len(s)-num:] for s in c]
				if info_tree["spec_filter"] in species:
					useful.append(c)
	else:
		useful = colection
	return useful

def check_subtrees_for_duplications(list_trees,tree_mode,species_position,orto_mode):
	alright = True
	for nw in list_trees:
		if type(nw) == str:
			t = load_tree(nw,tree_mode,species_position,0)
			c = t.get_leaf_names()
			dupli = check_duplications(c,tree_mode,species_position)
			if dupli:
				alright = False
	if not alright:
		if orto_mode == "m":
			sys.exit("Pruned trees were found that contained duplicated leaves, please re-define your duplication nodes or use an alternative orthology mode")
		else:
			sys.exit("Something went wrong with the subtree reconstruction")

def get_evol_events(tree,orto_mode,stree):
	if orto_mode == "s":
		tree = sm.annotate_tree(tree)
		events = get_manual_events(tree)
	elif orto_mode == "r":
		try:
			events = tree.reconcile(stree)[1]
		except:
			sys.exit("There was a problem while reconciling your tree to the species tree. The species tree needs to contain all the species contained in the gene tree and the species names should be the same. If you still have problems, please contact the administrator.")
	elif orto_mode == "m":
		events = get_manual_events(tree)
	return events

def get_manual_events(tree):
	#Obtains the needed information of a manual annotation of duplications
	events = []
	for node in tree.traverse():
		if "evoltype" in node.features:
			if node.evoltype == "D":
				ev = Manual_Events()
				ev.etype = "D"
				ev.in_seqs = node.get_children()[0].get_leaf_names()
				ev.out_seqs = node.get_children()[1].get_leaf_names()
				events.append(ev)
	return events

#Function that processes duplication events
def get_duplications (tree,orto_mode,stree):
	#This gets all the evolutive events that happen in the tree and keeps the sequences that happen on either side of a duplication, it also remembers the total number of sequences involved in each duplication
	events=get_evol_events(tree,orto_mode,stree)
	induplis=[]
	outduplis=[]
	llargada=[]
	for ev in events:
		if ev.etype == "D":
			inseqs=[]
			outseqs=[]
			for i in ev.in_seqs:
				inseqs.append(i)
			for i in ev.out_seqs:
				outseqs.append(i)
			inseqs.sort()
			outseqs.sort()
			induplis.append(inseqs)
			outduplis.append(outseqs)
			length=len(inseqs) + len(outseqs)
			llargada.append(length)
	return induplis,outduplis,llargada

def generate_orthology_lists(induplis, outduplis,tree,llargada,tree_mode,species_position,limit,orto_mode):
	#From a list of duplications (inseqs and outseqs), this generates lists of single copy, orthologous, species. The duplications are recorded in two lists, which contain the sequences at either side of the duplication. These are ordered considering the number of sequences involved in the duplication. Following this, for each duplication, we start deleting one side or the other from the trees, generating more trees as we go until they no longer have duplications inside
	tree_lists=[]
	to_add=[]
	completed=[]
	tree_lists.append(tree.get_leaf_names())
	list1,list2=sort_duplis(induplis, outduplis,llargada)
	list_dupli_species = []
	stop = False
	if orto_mode == "s":
		for i in list1:
			if not stop:
				if not tree_lists == []:
					index=list1.index(i)
					part1=list1[index]
					part2=list2[index]
					to_add=[]
					to_add,completed=process_lists(tree_lists,part1,to_add,completed,tree_mode,species_position)
					to_add,completed=process_lists(tree_lists,part2,to_add,completed,tree_mode,species_position)
					if limit != None and limit != 0:
						if len(completed) > limit:
							stop = True
					tree_lists=[]
					for add in to_add:
						if not add in tree_lists:
							tree_lists.append(add)
	else:
		#When the orthology prediction is based on reconciliation we can not base the partitioning on presence/abscence of duplicates so a different approach needs to be taken
		for i in list1:
			index = list1.index(i)
			part1 = list1[index]
			part2 = list2[index]
			llista_arbres = []
			removed_tree = []
			for t in tree_lists:
				remove = False
				result = ""
				if part1[0] in t:
					result=process_lists_reconciliation(t,part1)
					if result != "":
						remove = True
						llista_arbres.append(result)
					result=process_lists_reconciliation(t,part2)
					if result != "":
						remove = True
						llista_arbres.append(result)
					if remove:
						removed_tree.append(t)
			for t in removed_tree:
				tree_lists.remove(t)
			for t in llista_arbres:
				tree_lists.append(t)
			to_add = []
		completed = tree_lists
		if limit != None and limit != 0:
			if len(completed) > limit:
				stop = True
	completed.sort()
	return completed,stop

def process_lists_reconciliation(tree_list,part_del):
	result_tree = remove_seqs(tree_list,part_del)
	if result_tree == tree_list:
		result_tree = ""
	return result_tree

def sort_duplis (induplis, outduplis, llargada):
	#This sorts the duplications from the most ancient to the most recent
	list1=[]
	list2=[]
	while not len(llargada)==0:
		index = llargada.index(max(llargada))
		list1.append(induplis[index])
		list2.append(outduplis[index])
		del llargada[llargada.index(max(llargada))]
		del induplis[index]
		del outduplis[index]
	return list1,list2


def process_lists (tr_list,part_del,list_add,completed,tree_mode,species_position):
	# For each tree in a list of trees, a list of species are deleted from each tree (if present). the list of remaining leaves are then checked for duplication events
	for tree in tr_list:
		result_tree=remove_seqs(tree,part_del)
		result_tree.sort()
		dup=check_duplications(result_tree,tree_mode,species_position)
		if dup:
			list_add.append(result_tree)
		else:
			completed.append(result_tree)

	return list_add, completed

def remove_seqs(tree,part_del):
	# The actual part that deletes the leaves of the tree that are not on the list
	new_tree=[]
	for i in tree:
		if not i in part_del:
			new_tree.append(i)
	return new_tree
		
def check_duplications(tree,tree_mode,species_position):
	#Checks whether there are duplications in a list of species
	tree.sort()
	dupli=False
	old=""
	for i in tree:
		spe=get_species_name(i,tree_mode,species_position)
		if old == spe:
			dupli= True
		else:
			old=spe
	return dupli

def get_trees(ortholog,nw,tree_mode,species_position):
	loners=[]
	#From a list of leaves this will gestion the creation of the trees following the example of an original tree
	list_newick=[]
	for leaves in ortholog:
		if len(leaves)>2:
			nw1=create_tree_from_leaves(leaves,nw,tree_mode,species_position)
			if not nw1 == "":
				list_newick.append(nw1)
		else:
			list_newick.append(leaves)
	return list_newick

def create_tree_from_leaves(leaves,nw,tree_mode,species_position):
	#This part is the one that reconstructs a tree from a list of leaves, it actually removes all the leaves that are not in the list
	tree = load_tree(nw,tree_mode,species_position,0)
	tr_leafs=tree.get_leaf_names()
	for leaf in tr_leafs:
		if not leaf in leaves:
			l=tree.get_leaves_by_name(leaf)
			l=l[0]
			lup=l.up
			lup.remove_child(l)
			lup.delete()
	#Sometimes two nodes remain at the root, in order to remove one, we need to detach it
	root=tree.get_tree_root()
	childs=root.get_children()
	if len(childs)<2:
		child=childs[0]
		tree=child.detach()
	nw1=""
	if not len(tree.get_children())<2:
		nw1=tree.write()
	return nw1

def add_distances (tree):
	#The tree needs to have distances, so we force them to be 1 if they are not there
	for n in tree.get_descendants():
		if (n.dist == 0):
			n.dist = 1
	return tree

def get_species_name(node_name_string,tree_mode,species_position):
	
	if tree_mode == "s":
		spe = node_name_string[0:3]
	else:
		num = tree_mode.replace("s","")
		if num == "all":
			spe = node_name_string
		elif num == "phylomedb":
			cs = node_name_string.split("_")
			if len(cs) > 1:
				spe = cs[1]
			else:
				spe = cs[0]
		else:
			if species_position == "1":
				if num == "_":
					spe = node_name_string.split("_")[0]
				elif num == "-":
					spe = node_name_string.split("-")[0]
				elif num == "#":
					spe = node_name_string.split("#")[0]
				elif num == ".":
					spe = node_name_string.split(".")[0]
				else:
					spe = node_name_string[0:int(num)]
			elif species_position == "-1":
				if num == "_":
					spe = node_name_string.split("_")[-1]
				elif num == "-":
					spe = node_name_string.split("-")[-1]
				elif num == "#":
					spe = node_name_string.split("#")[-1]
				elif num == ".":
					spe = node_name_string.split(".")[-1]
				else:
					spe = node_name_string[len(node_name_string) - int(num):]
	return spe

def check_root(tree):
	#Deals with the problem ete has of leaving a root with only one child
	root = tree.get_tree_root()
	if len(root.get_children()) < 2:
		child = root.get_children()[0]
		tree = child.detach()
	return tree
	
#The function loads a tree depending on whether its a protein or a species tree
def load_tree(nw,tree_mode,species_position,format):
	tree = ete2.PhyloTree(nw,format = format)
	if tree_mode == "s" and species_position == "-1":
		tree.set_species_naming_function(set_species_name0_end)
	elif tree_mode != "s" and species_position == "1":
		num = tree_mode.replace("s","")
		try:
			if num == "1":
				tree.set_species_naming_function(set_species_name1)
			elif num == "2":
				tree.set_species_naming_function(set_species_name2)
			elif num == "4":
				tree.set_species_naming_function(set_species_name4)
			elif num == "5":
				tree.set_species_naming_function(set_species_name5)
			elif num == "6":
				tree.set_species_naming_function(set_species_name6)
			elif num == "7":
				tree.set_species_naming_function(set_species_name7)
			elif num == "_":
				tree.set_species_naming_function(set_species_name8)
			elif num == "-":
				tree.set_species_naming_function(set_species_name9)
			elif num == "#":
				tree.set_species_naming_function(set_species_name10)
			elif num == ":":
				tree.set_species_naming_function(set_species_name11)
			elif num == ".":
				tree.set_species_naming_function(set_species_name12)
			elif num == "all":
				tree.set_species_naming_function(set_species_name13)
			elif num == "phylomedb":
				tree.set_species_naming_function(set_species_name14)
		except:
			print "Treeko was unable to define species codes using the delimiters provided"
			exit()
	elif tree_mode != "s" and species_position == "-1":
		num = tree_mode.replace("s","")
		try:
			if num == "1":
				tree.set_species_naming_function(set_species_name1_end)
			elif num == "2":
				tree.set_species_naming_function(set_species_name2_end)
			elif num == "4":
				tree.set_species_naming_function(set_species_name4_end)
			elif num == "5":
				tree.set_species_naming_function(set_species_name5_end)
			elif num == "6":
				tree.set_species_naming_function(set_species_name6_end)
			elif num == "7":
				tree.set_species_naming_function(set_species_name7_end)
			elif num == "_":
				tree.set_species_naming_function(set_species_name8_end)
			elif num == "-":
				tree.set_species_naming_function(set_species_name9_end)
			elif num == "#":
				tree.set_species_naming_function(set_species_name10_end)
			elif num == ":":
				tree.set_species_naming_function(set_species_name11_end)
			elif num == ".":
				tree.set_species_naming_function(set_species_name12_end)
			elif num == "all":
				tree.set_species_naming_function(set_species_name13)
			elif num == "phylomedb":
				tree.set_species_naming_function(set_species_name14)
		except:
			print "Treeko was unable to define species codes using the delimiters provided"
			exit()
	return tree

################################################################################################################
#Different species setting modes
################################################################################################################

# species_position = 1
def set_species_name1(leaf_name):
	if len(leaf_name) < 1:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	spcode = leaf_name[0:1]
	return spcode

def set_species_name2(leaf_name):
	if len(leaf_name) < 2:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	spcode = leaf_name[0:2]
	return spcode

def set_species_name4(leaf_name):
	if len(leaf_name) < 4:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[0:4]

def set_species_name5(leaf_name):
	if len(leaf_name) < 5:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	spcode = leaf_name[0:5]
	return spcode

def set_species_name6(leaf_name):
	if len(leaf_name) < 6:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[0:6]

def set_species_name7(leaf_name):
	if len(leaf_name) < 7:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[0:7]

def set_species_name8(leaf_name):
	return leaf_name.split("_")[0]

def set_species_name9(leaf_name):
	return leaf_name.split("-")[0]

def set_species_name10(leaf_name):
	return leaf_name.split("#")[0]

def set_species_name11(leaf_name):
	return leaf_name.split(":")[0]

def set_species_name12(leaf_name):
	return leaf_name.split(".")[0]

def set_species_name13(leaf_name):
	return leaf_name
	
def set_species_name14(leaf_name):
	name = leaf_name.name.split("_")
	if len(name) > 1:
		name = name[1]
	else:
		name = name[0]
	return name

# species_position = -1
def set_species_name1_end(leaf_name):
	if len(leaf_name) < 1:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	spcode = leaf_name[len(leaf_name)-1:]
	return spcode

def set_species_name2_end(leaf_name):
	if len(leaf_name) < 2:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	spcode = leaf_name[len(leaf_name)-2:]
	return spcode

def set_species_name0_end(leaf_name):
	if len(leaf_name) < 3:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	spcode = leaf_name[len(leaf_name)-3:]
	return spcode

def set_species_name4_end(leaf_name):
	if len(leaf_name) < 4:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[len(leaf_name)-4:]

def set_species_name5_end(leaf_name):
	if len(leaf_name) < 5:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[len(leaf_name)-5:]

def set_species_name6_end(leaf_name):
	if len(leaf_name) < 6:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[len(leaf_name)-6:]

def set_species_name7_end(leaf_name):
	if len(leaf_name) < 7:
		print "One or more of the leaves in your tree is shorter than the species limit provided, please, provide a shorter limit"
		exit()
	return leaf_name[len(leaf_name)-7:]

def set_species_name8_end(leaf_name):
	return leaf_name.split("_")[-1]

def set_species_name9_end(leaf_name):
	return leaf_name.split("-")[-1]

def set_species_name10_end(leaf_name):
	return leaf_name.split("#")[-1]

def set_species_name11_end(leaf_name):
	return leaf_name.split(":")[-1]

def set_species_name12_end(leaf_name):
	return leaf_name.split(".")[-1]

def improve_outgroup_prediction(current_outgroup,spec):
	current_age = spec[list(current_outgroup.get_species())[0]]
	same_age = True
	while same_age:
		sisters = current_outgroup.get_sisters()
		for sis in sisters:
			if current_age == sis.get_age(spec):
				for l in list(sis.get_species()):
					if spec[l] != current_age:
						same_age = False
			else:
				same_age = False
		if same_age:
			if current_outgroup.is_root():
				same_age = False
			else:
				current_outgroup = current_outgroup.up
	return current_outgroup
	
	
def set_chosen_outgroup (tree,kroot,spec,tree_mode,species_position,root_spec,orto_mode,stree):
	#Read a tree, looks if it's still a file or string and then places the root
	if type(tree) is str:
		tree = load_tree(tree,tree_mode,species_position,0)
	tree = add_distances(tree)
	if kroot == "m":
		#This places the root at midpoint
		outgroup = tree.get_midpoint_outgroup()
	elif kroot == "d":
		outgroup = root_min_dupli(tree,orto_mode,stree,tree_mode,species_position)
	elif kroot == "f":
		if "s" in tree_mode:
			all_present = True
			for species in tree.get_species():
				if species not in spec:
					spec[species] = "1"
					all_present = False
			if not all_present:
				print "WARNING: not all the species in your tree were present in the anotation file, a default value of 1 has been assigned as their age value"
			try:
				outgroup = tree.get_farthest_oldest_leaf(spec)
			except:
				print "Some species may not be present in your configuration file, please, make sure all are there before running treeKo again"
				exit()
		else:
			print "The tree can not be rooted using the farthest species when the tree mode has been set to protein, please, use one of the other methods to root the tree"
			exit()
	elif kroot == "fi":
		if "s" in tree_mode:
			all_present = True
			for species in tree.get_species():
				if species not in spec:
					spec[species] = 1
					all_present = False
			if not all_present:
				print "WARNING: not all the species in your tree were present in the anotation file, a default value of 1 has been assigned as their age value"
			#try:
			tree = root_at_youngest_leaf(tree,spec)
			outgroup = tree.get_farthest_oldest_leaf(spec)
			outgroup = improve_outgroup_prediction(outgroup,spec)
			#except:
				#print "Some species may not be present in your configuration file, please, make sure all are there before running treeKo again"
				#exit()
		else:
			print "The tree can not be rooted using the farthest species when the tree mode has been set to protein, please, use one of the other methods to root the tree"
			exit()
	elif kroot == "s":
		if tree_mode == "p":
			try:
				outgroup = tree.get_leaves_by_name(root_spec)[0]
			except:
				print "The tree could not be rooted by this protein, check that your chosen protein is present in both trees"
				exit()
		else:
			try:
				possible_outgroups = []
				for leaf in tree.get_leaves():
					if root_spec == leaf.species:
						possible_outgroups.append(leaf.name)
				possible_outgroups.sort()
				outgroup = tree.get_leaves_by_name(possible_outgroups[0])[0]
			except:
				print "The tree could not be rooted by this species, check that your chosen species is present in both trees"
				exit()
	elif kroot == "n":
		if len(tree.get_children()) != 2:
			print "One of the trees is not rooted, please, change the rooting option"
			exit()
	try:
		if kroot != "n":
			try:
				tree.set_outgroup(outgroup)
			except:
				tree.set_outgroup(tree.get_leaves()[0])
				tree.set_outgroup(outgroup)
	except:
		print "There was a problem while rooting one of the trees, please change your rooting option or contact us."
		tree = "NOTROOTED"
	#t = sm.annotate_tree(tree)
	#d = len(t.search_nodes(evoltype = "D"))
	#m = len(t.search_nodes(evoltype = "MD"))
	#print d,m,d+m
	#tree.show()
	return tree

def root_at_youngest_leaf(tree,spec):
	trobat = False
	for sp in spec:
		if not trobat:
			n = tree.search_nodes(species=sp)
			if len(n) > 0:
				tree.set_outgroup(n[0])
				trobat = True
	return tree

def rename_leaves(tree):
	list_names = set(tree.get_leaf_names())
	for name in list_names:
		i = 1
		for leaf in tree.get_leaves_by_name(name):
			leaf.name = leaf.name + str(i)
			i = i + 1
	return tree

def most_distant(nw, possible_roots,tree_mode,species_position):
	distances = {}
	for r in possible_roots:
		tree = load_tree(nw,tree_mode,species_position,1)
		node = tree.search_nodes(node_name = r)[0]
		for leaf in tree.get_leaves():
			try:
				d = node.get_distance(leaf)
				if r not in distances:
					distances[r] = d
				else:
					if d > distances[r]:
						distances[r] = d
			except:
				pass
	maxim = ""
	outgroup = possible_roots[0]
	for r,d in distances.iteritems():
		if maxim == "" or d > maxim:
			maxim = d
			outgroup = r
	return outgroup
           
def name_nodes(tree):
	i = 0
	for node in tree.traverse():
		name = "n"+str(i)
		node.add_features(node_name = name)
		i = i + 1
	return tree,i
	
def root_min_dupli(tree,orto_mode,stree,tree_mode,species_position):
	tree,num_nodes = name_nodes(tree)
	nw = tree.write(features = ["node_name"])
	root = {}
	minim = ""
	i = 0
	while i < num_nodes:
		t = load_tree(nw,tree_mode,species_position,1)
		name = "n"+str(i)
		node = t.search_nodes(node_name = name)[0]
		if not node.is_root():
			t.set_outgroup(node)
			if orto_mode == "s":
				t = sm.annotate_tree(t)
				duplis = len(t.search_nodes(evoltype = "D"))
				mduplis = len(t.search_nodes(evoltype = "MD"))
				number = duplis + mduplis
			elif orto_mode == "r":
				rtree = t.reconcile(stree)[0]
				number = len(t.search_nodes(evoltype = "D"))
			else:
				sys.exit("If the manual ortology descritpion mode was chosen, the tree should be rooted")
			root[name] = number
			if number < minim or minim == "":
				minim = number
		i = i + 1
	possible_roots = [m for m in root if root[m] == minim]
	possible_roots.sort()

	if len(possible_roots) == 1:
		outgroup = tree.search_nodes(node_name = possible_roots[0])[0]
		tree.set_outgroup(outgroup)
	else:
		outgroup = most_distant(nw,possible_roots,tree_mode,species_position)
		outgroup = tree.search_nodes(node_name = outgroup)[0]
	return outgroup

