#!/usr/bin/env python

# ########################################################################
#
# Copyright (C) 2011 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es, tgabaldon[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################

import ete2,sys
from optparse import OptionParser
import tree_comparison as tc
import reconciliation_distance as rd
import bootstrap_support as bs
import os,os.path,random
import phylome_support as ps
import HGT_treeKO as HGT
import subtree_reconstruction as sr

def custumize_parser():
	parser = OptionParser(usage = '\n treeKo.py -p program_to_use -a file_tree1 [-b file_tree2 | -l tree_list] [Options]\n',\
					version = "TreeKo_v1 2011",\
					description = "The treeKO package presents different tree comparison measures.")
	
	parser.add_option('-p', '--program', action='store', dest='program', help='Which program of the treeKO package you wish to use (tc=Tree comparison, ps=Phylome support, rd=Reconciliation distance, bs = Bootstrap support, ht = HGT specific, st = Obtain subtrees)')
	parser.add_option('-a', '--treea', action='store', dest='file1', help='file where the first tree is stored, this file is needed')
	parser.add_option('-b', '--treeb', action='store', dest='file2', help='file where the second tree is stored, this file in needed')
	parser.add_option('-l', '--list_trees', action='store', dest='list_trees', help="list of tree paths for multiple comparison against one reference tree")
	parser.add_option('-c','--conf',action='store',dest='conf_file',help='A configuration file is desirable to run this program, default values will be assumed otherwise. For more information refer to the manual provided with the program.')
	parser.add_option('-f','--filter',action='store',dest='seed_filter',help="Filters pruned trees so only those with the seed are used (tc)")
	parser.add_option('-o', '--out', action='store', dest='output', help='outfile (tc/ps/rd)')
	parser.add_option('-t', '--outtree', action='store', dest='outtree', help='species tree outfile (ps)')
	parser.add_option('-n', '--outnw', action='store', dest='outnw', help='newick tree outfile (ps)')
	
	return parser

def check_output_file(options_output):
	if options_output:
		try:
			outfile = open(options_output,"w")
		except:
			print "TreeKo was unable to open your chosen outfile"
			exit()
	else:
		outfile = sys.stdout
	return outfile
	

def main():
	#Parse options and launch programs
	parser = custumize_parser()
	(options,args) = parser.parse_args()
	if not options.program:
		sys.exit(parser.print_help())
	
	outfile = check_output_file(options.output)
	if options.program == "st":
		if not options.file1 and not options.list_trees:
			sys.exit(parser.print_help())
		if options.file1 and options.list_trees:
			sys.exit("The -a and the -l option are not compatible, please choose one of the two inputs")
		if options.file1:
			if not os.path.exists(options.file1):
				sys.exit("TreeKO was unable to find the tree file, please check your command line")
			if options.seed_filter:
					seed = options.seed_filter
			else:
				seed = None
			file2 = options.file1
			nw1,nw2,is_rooted1,is_rooted2,check_species1,check_species2 = tc.check_input_files(options.file1,file2)
			default,general_info,info_tree1,info_tree2,print_info = tc.parse_configuration_file(options.conf_file,outfile,is_rooted1,is_rooted2,seed,False)
			colection,total_tree_length,deli,stop=sr.make_subtrees(options.file1,general_info,info_tree1,check_species1)
			tc.print_subtrees(outfile,options.file1,None,colection,None)
		elif options.list_trees:
			if not os.path.exists(options.list_trees):
				sys.exit("TreeKo was unable to find the list of trees, please check your command line")
			for line in open(options.list_trees):
				line = line.strip()
				files = line.split("\t")
				if len(files) == 2:
					if options.seed_filter:
						seed = options.seed_filter
					else:
						seed = None
					if "(" in files[1] and ")" in files[1] and ":" in files[1]:
						file = open("tmp_tree.txt","w")
						print >>file,files[1]
						file.close()
						file1 = "tmp_tree.txt"
					else:
						file1 = files[1]
				else:
					if "(" in line and ")" in line and ":" in line:
						file = open("tmp_tree.txt","w")
						print >>file, line
						file.close()
						file1 = "tmp_tree.txt"
					else:
						file1 = line
					seed = None
				file2 = file1
				nw1,nw2,is_rooted1,is_rooted2,check_species1,check_species2 = tc.check_input_files(file1,file2)
				default,general_info,info_tree1,info_tree2,print_info = tc.parse_configuration_file(options.conf_file,outfile,is_rooted1,is_rooted2,seed,False)
				colection,total_tree_length,deli,stop=sr.make_subtrees(file1,general_info,info_tree1,check_species1)
				colection = tc.filter_colection(colection,general_info,info_tree1)
				tc.print_subtrees(outfile,file1,None,colection,None)
	elif options.program == "tc":
		if not options.file1 or (not options.file2 and not options.list_trees):
			sys.exit(parser.print_help())
		if options.file2 and options.list_trees:
			sys.exit("The -b and the -l option are not compatible, choose one of the two inputs to be compared to your reference tree")
		if options.file2:
			if not os.path.exists(options.file1):
				sys.exit("TreeKO was unable to find the first tree file, please check your command line")
			if not os.path.exists(options.file2):
				sys.exit("TreeKO was unable to find the second tree file, please check your command line")
			if options.seed_filter:
				seed = options.seed_filter
			else:
				seed = None
			presence_hgt1 = HGT.check_for_HGT(options.file1)
			presence_hgt2 = HGT.check_for_HGT(options.file2)
			if not presence_hgt1 and not presence_hgt2:
				d = tc.tree_comparison(options.file1,options.file2,options.conf_file,outfile,seed)
			else:
				print "\n***WARNING: Horizontal gene transfer events have been annotated in your tree, treeKO will proceed to split the tree by those events.\n If you do want to run a normal tree comparison, please remove the &&NHX HGT annotations. For more information check the manual***\n"
				HGT.launch_hgt_run(options.file1,options.file2,presence_hgt1,presence_hgt2,options.conf_file,seed,outfile,"tc")
		else:
			if not os.path.exists(options.list_trees):
				sys.exit("TreeKo was unable to find the list of trees, please check your command line")
			presence_hgt1 = HGT.check_for_HGT(options.file1)
			for line in open(options.list_trees):
				line = line.strip()
				files = line.split("\t")
				if len(files) == 2:
					if options.seed_filter:
						seed = options.seed_filter
					else:
						seed = None
					if "(" in files[1] and ")" in files[1] and ":" in files[1]:
						file = open("tmp_tree.txt","w")
						print >>file,files[1]
						file.close()
						file2 = "tmp_tree.txt"
					else:
						file2 = files[1]
				else:
					if "(" in line and ")" in line and ":" in line:
						file = open("tmp_tree.txt","w")
						print >>file, line
						file.close()
						file2 = "tmp_tree.txt"
					else:
						file2 = line
					seed = None
				try:
					presence_hgt2 = HGT.check_for_HGT(file2)
					if not presence_hgt1 and not presence_hgt2:
						d = tc.tree_comparison(options.file1,file2,options.conf_file,outfile,seed)
					else:
						print "WARNING: Horizontal gene transfer events have been annotated in your tree, treeKO will proceed to split the tree by those events. If you do want to run a normal tree comparison, please remove the &&NHX HGT annotations. For more information check the manual"
						HGT.launch_hgt_run(options.file1,file2,presence_hgt1,presence_hgt2,options.conf_file,seed,outfile,"tc")
				except:
					print "Unable to compare the tree: ",line
	elif options.program == "rd":
		if not options.file1 and (not options.file2 or not options.list_trees):
			sys.exit(parser.print_help())
		if not os.path.exists(options.file1):
			sys.exit("TreeKO was unable to find the species tree file, please check your command line")
		if options.file2 and options.list_trees:
			sys.exit("The -b and the -l option are not compatible, choose one of the two inputs to be compared to your reference tree")
		if options.file2:
			if not os.path.exists(options.file2):
				sys.exit("TreeKO was unable to find the second tree file, please check your command line")
			presence_hgt1 = HGT.check_for_HGT(options.file1)
			presence_hgt2 = HGT.check_for_HGT(options.file2)
			if not presence_hgt1 and not presence_hgt2:
				a,b = rd.reconciliation_distance(options.file1,options.file2,options.conf_file,options.output,outfile)
			else:
				print "\n***WARNING: Horizontal gene transfer events have been annotated in your tree, treeKO will proceed to split the tree by those events.\n If you do want to run a normal tree comparison, please remove the &&NHX HGT annotations. For more information check the manual***\n"
				HGT.launch_hgt_run(options.file1,options.file2,presence_hgt1,presence_hgt2,options.conf_file,seed,outfile,"rd")
		else:
			if not os.path.exists(options.file1):
				sys.exit("TreeKO was unable to find the species tree file, please check your command line")
			if not os.path.exists(options.list_trees):
				sys.exit("TreeKo was unable to find the list of trees, please check your command line")
			presence_hgt1 = HGT.check_for_HGT(options.file1)
			for line in open(options.list_trees):
				line = line.strip()
				files = line.split("\t")
				if len(files) == 2:
					if "(" in files[1] and ")" in files[1] and ":" in files[1]:
						file = open("tmp_tree.txt","w")
						print >>file,files[1]
						file.close()
						file2 = "tmp_tree.txt"
					else:
						file2 = files[1]
				else:
					if "(" in line and ")" in line and ":" in line:
						file = open("tmp_tree.txt","w")
						print >>file, line
						file.close()
						file2 = "tmp_tree.txt"
					else:
						file2 = line
				try:
					presence_hgt2 = HGT.check_for_HGT(file2)
					if not presence_hgt1 and not presence_hgt2:
						a,b = rd.reconciliation_distance(options.file1,file2,options.conf_file,options.output,outfile)
					else:
						print "\n***WARNING: Horizontal gene transfer events have been annotated in your tree, treeKO will proceed to split the tree by those events.\n If you do want to run a normal tree comparison, please remove the &&NHX HGT annotations. For more information check the manual***\n"
						HGT.launch_hgt_run(options.file1,file2,presence_hgt1,presence_hgt2,options.conf_file,seed,outfile,"rd")
				except:
					print "Unable to compare the tree: ",line
	elif options.program == "ps":
		if not options.file1 and (not options.file2 or not options.list_trees):
			sys.exit(parser.print_help())
		if not options.outnw:
			outnw = "species_tree.nw"
		else:
			outnw = options.outnw
		if not options.outtree:
			outtree = "species_tree.pdf"
		else:
			outtree = options.outtree
			if outtree[-4:] != ".pdf":
				outtree = outtree+".pdf"
		if options.file2:
			sys.exit("This input is not available for this program, please provide a list of trees")
		if options.list_trees:
			if not os.path.exists(options.list_trees):
				sys.exit("TreeKO was unable to find the tree list file, please check your command line")
			else:
				ps.phylome_support(options.file1,options.list_trees,options.conf_file,outfile,outtree,outnw)
	elif options.program == "bs":
		if not options.file1 or (not options.file2 or not options.list_trees):
			sys.exit(parser.print_help())
		if not options.outnw:
			outnw = "species_tree.nw"
		else:
			outnw = options.outnw
		if not options.outtree:
			outtree = "species_tree.pdf"
		else:
			outtree = options.outtree
			if outtree[-4:] != ".pdf":
				outtree = outtree+".pdf"
		if options.file2:
			sys.exit("This input is not available for this program, please provide a list of trees")
		if options.list_trees:
			if not os.path.exists(options.list_trees):
				sys.exit("TreeKO was unable to find the tree list file, please check your command line")
			else:
				bs.bootstrap_support(options.file1,options.list_trees,"",options.conf_file,outtree,outnw)
	outfile.close()

if __name__ == '__main__':
	main()
