#!/usr/bin/env python


# ########################################################################
#
# Copyright (C) 2010 by Marina Marcet Houben and Toni Gabaldon. All rights 
# reserved.  
# email: mmarcet[at]crg.es
#
# TreeKO is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# TreeKO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# ########################################################################


import re,sys,datetime,ete2,os,os.path
from optparse import OptionParser
from random import randint
import subtree_reconstruction as sr

#########################################################################################################################################
# Basic functions
#########################################################################################################################################

def load_initial_data(d):
	if d == "g":
		info = {"tree_mode" : "s", "species_position" : "1", "orto_mode" : "s", "species2age" : {}, "species_tree" : "", "root_species" : "", "limit" : 0, "overlap" : False}
	elif d == "p":
		info = {"pall" : False, "psub" : False, "pbiol" : False, "pstr" : False, "pconfig" : False, "ptime" : False, "verbose" : False, "oneline": False}
	else:
		info = {"kind_root" : "m", "spec_filter" : None, "prot_filter" : None, "bootstrap" : None}
	return info
		
def parse_conf_file(file,is_rooted1,is_rooted2,seed):
	
	#Initialize data:
	print_info = load_initial_data("p")
	general_info = load_initial_data("g")
	info_tree1 = load_initial_data("t")
	info_tree2 = load_initial_data("t")
	species_limit = "3"
	
	#Read configuration file
	for line in open(file):
		line = line.strip()
		if line != "":
			if line[0] != "#":
				if " " in line:
					dad = line.split(" ")
				else:
					dad = line.split("\t")
				if dad[0].lower() == "tree_mode":
					general_info["tree_mode"] = dad[1].lower()
				elif dad[0].lower() == "species_position":
					general_info["species_position"] = dad[1].lower()
				elif dad[0].lower() == "root_method1":
					kind_root1 = dad[1].lower()
					if kind_root1 != "f" and kind_root1 != "s" and kind_root1 != "m" and kind_root1 != "n" and kind_root1 != "d" and kind_root1 != "fi":
						sys.exit("The root method provided is not supported, please check the manual for the proper configuration")
					if is_rooted1:
						info_tree1["kind_root"] = "n"
					else:
						info_tree1["kind_root"] = kind_root1
				elif dad[0].lower() == "root_method2":
					kind_root2 = dad[1].lower()
					if kind_root2 != "f" and kind_root2 != "s" and kind_root2 != "m" and kind_root2 != "n" and kind_root2 != "d" and kind_root2 != "fi":
						sys.exit("The root method provided is not supported, please check the manual for the proper configuration")
					if is_rooted2:
						info_tree2["kind_root"] = "n"
					else:
						info_tree2["kind_root"] = kind_root2
					
				elif dad[0].lower() == "spec_filter1":
					info_tree1["spec_filter"] = dad[1]
				elif dad[0].lower() == "spec_filter2":
					info_tree2["spec_filter"] = dad[1]
				elif dad[0].lower() == "prot_filter1":
					info_tree1["prot_filter"] = dad[1]
				elif dad[0].lower() == "prot_filter2":
					info_tree2["prot_filter"] = dad[1]
				
				elif dad[0].lower() == "species_limit":
					species_limit = dad[1].lower()
					if species_limit != "1" and species_limit != "2" and species_limit != "3" and species_limit != "4" and species_limit != "5" and species_limit != "6" and species_limit != "7" and species_limit != "_" and species_limit != "-" and species_limit != "#" and species_limit != ":" and species_limit != "." and species_limit != "all" and species_limit != "phylomedb":
						print "The species delimiter is not accepted. Please, check the manual for accepted limits for the species name"
						exit()

				elif dad[0].lower() == "root_data":
					if ":" in dad[1]:
						general_info["species2age"][dad[1].split(":")[0]] = int(dad[1].split(":")[1])
					else:
						sys.exit("Information should be provided in the following format:\nspecies_name:level")

				elif dad[0].lower() == "root_species":
					general_info["root_species"] = dad[1]

				elif dad[0].lower() == "orto_mode":
					orto_mode = dad[1].lower()
					if orto_mode != "s" and orto_mode != "r" and orto_mode != "m":
						sys.exit("The orthology detection mode is not valid, please revise the information provided by orto_mode")
					general_info["orto_mode"] = orto_mode

				elif dad[0].lower() == "species_tree":
					general_info["species_tree"] = dad[1]
					
				elif dad[0].lower() == "support1":
					try:
						info_tree1["bootstrap"] = float(dad[1])
					except:
						sys.exit("There was a problem with the support value for tree 1, please check your configuration file")
				elif dad[0].lower() == "support2":
					try:
						info_tree2["bootstrap"] = float(dad[1])
					except:
						sys.exit("There was a problem with the support value for tree 2, please check your configuration file")

				elif dad[0].lower() == "limit":
					try:
						general_info["limit"] = int(dad[1])
					except:
						print "Limit has to have a numerical value, put 0 if you do not whish to use any limit"
						exit()
						
				elif dad[0].lower() == "overlap":
					general_info["overlap"] = True
				elif dad[0].lower() == "print_all":
					print_info["pall"] = True
				elif dad[0].lower() == "print_pruned_trees":
					print_info["psub"] = True
				elif dad[0].lower() == "print_speciation_distance":
					print_info["pbiol"] = True
				elif dad[0].lower() == "print_strict_distance":
					print_info["pstr"] = True 
				elif dad[0].lower() == "print_configuration":
					print_info["pconfig"] = True
				elif dad[0].lower() == "print_time":
					print_info["ptime"] = True
				elif dad[0].lower() == "verbose":
					print_info["verbose"] = True
				elif dad[0].lower() == "print_oneline":
					print_info["oneline"] = True
				else:
					print "WARNING: The information provided in the configuration file is not in a recognised format, please check the manual for further information."
					print line
	#Checking necessary info
	if general_info["orto_mode"] == "r":
		if general_info["species_tree"] != "":
			if not os.path.exists(general_info["species_tree"]):
				sys.exit("Unable to find the species tree, please check your configuration file")
		else:
			sys.exit("A species tree needs to be provided if reconciliation is chosen as the orthology prediction program, please provide the path to the species tree in the configuration file")
		if info_tree2["bootstrap"] != None or info_tree1["bootstrap"] != None:
			sys.exit("At the moment multifurcated trees cannot be treated with tree reconciliation methods, please switch to species overlap or remove the support limits")
	if general_info["species_position"] != "1" and general_info["species_position"] != "-1":
		sys.exit("The species position has to be either 1 or -1, please check your configuration file")
	if info_tree1["kind_root"] != "n":
		if info_tree1["kind_root"] == "":
			info_tree1["kind_root"] = "m"
		elif info_tree1["kind_root"] == "f" or info_tree1["kind_root"] == "fi":
			if general_info["species2age"] == {}:
				sys.exit("The information for rooting at the farthest node is not provided, please check the manual for further information")
		elif info_tree1["kind_root"] == "s":
			if general_info["root_species"] == "":
				sys.exit("The information needed to root at a given protein is not provided, please check the manual for further information")
				exit()
		elif info_tree1["kind_root"] == "n" and not is_rooted1:
			sys.exit("Tree1 is not rooted, please provide a rooted tree or change your configuration file")
			
	if info_tree2["kind_root"] != "n":
		if info_tree2["kind_root"] == "":
			info_tree2["kind_root"] = "m"
		elif info_tree2["kind_root"] == "f" or info_tree2["kind_root"] == "fi":
			if general_info["species2age"] == {}:
				sys.exit("The information for rooting at the farthest node is not provided, please check the manual for further information")
		elif info_tree2["kind_root"] == "s":
			if general_info["root_species"] == "":
				sys.exit("The information needed to root at a given protein is not provided, please check the manual for further information")
		elif info_tree2["kind_root"] == "n" and not is_rooted2:
			sys.exit("Tree2 is not rooted, please provide a rooted tree or change your configuration file")
	if general_info["tree_mode"] == "":
		general_info["tree_mode"] = "s"
	
	if species_limit != "3":
		general_info["tree_mode"] = "s"+species_limit
	
	if not print_info["psub"] and not print_info["pbiol"] and not print_info["pstr"] and not print_info["pall"] and not print_info["oneline"]:
		print_info["pall"] = True
	
	if seed != None and info_tree2["prot_filter"] == None:
		info_tree2["prot_filter"] = seed
	elif info_tree2["prot_filter"] != None and info_tree2["prot_filter"] != seed:
		sys.exit("Protein filter supplied in the configuration file contradicts the -f option, please use only one of the options")
	
	if seed != None and info_tree1["prot_filter"] == None:
		info_tree1["prot_filter"] = seed
	elif info_tree1["prot_filter"] != None and info_tree1["prot_filter"] != seed:
		sys.exit("Protein filter supplied in the configuration file contradicts the -f option, please use only one of the options")
	
	if print_info["pall"]:
		print_info["pconfig"] = True
		print_info["psub"] = False
		print_info["pstr"] = False
		print_info["pbiol"] = False
		print_info["verbose"] = True
		print_info["oneline"] = False
	if print_info["oneline"] and not print_info["pall"]:
		print_info["pall"] = False
		print_info["pconfig"] = False
		print_info["psub"] = False
		print_info["pstr"] = False
		print_info["pbiol"] = False
		print_info["verbose"] = True

	return general_info,info_tree1,info_tree2,print_info


# Checks whether there is any overlap between the species in the two trees

def check_overlap(file1,file2,tree_mode,species_position):
	tree1 = sr.load_tree(file1,tree_mode,species_position,0)
	tree2 = sr.load_tree(file2,tree_mode,species_position,0)
	spe1 = tree1.get_species()
	spe2 = tree2.get_species()
	intersection = []
	intersection = list(spe1.intersection(spe2))
	if intersection == []:
		launch_treeKo = False
	else:
		launch_treeKo = True

	return launch_treeKo

#This calculates the time it took for the program to complete the work
def calc_time (now,outfile):
	after = datetime.datetime.now()
	difference = after - now
	minutes, seconds = divmod(difference.seconds, 60)
	hours, minutes = divmod(minutes, 60)
	print >>outfile,"TreeKO took: %d days, %d h, %d min, %d sec" % (difference.days, hours, minutes, seconds)


#Deals with the problem ete has of leaving a root with only one child
def check_root(tree):
	root = tree.get_tree_root()
	if len(root.get_children()) < 2:
		child = root.get_children()[0]
		tree = child.detach()
	return tree
	
#The tree needs to have distances, so we force them to be 1 if they are not there
def add_distances (tree):
	for n in tree.get_descendants():
		if (n.dist == 0):
			n.dist = 1
	return tree

#This function replaces the leaf names of a tree with their species codes.
def change_leaves (tree):
	for l in tree.get_leaves():
		l.name=l.species
	return tree

def getMinAndIndex(lst):
	minval,minidx = lst[0],0
	for i,v in enumerate(lst[1:]):
		if v < minval:
			minval,minidx = v,i+1
	return minval,minidx

def rename_leaves(tree):
	list_names = set(tree.get_leaf_names())
	for name in list_names:
		i = 1
		for leaf in tree.get_leaves_by_name(name):
			leaf.name = leaf.name + str(i)
			i = i + 1
	return tree

#########################################################################################################################################
# Tree comparison functions
#########################################################################################################################################

def compare_trees(col1,col2,tlen1,tlen2,deli1,deli2,tree_mode,species_position):
	#Main program used to compare trees
	# col1, col2 == colection of trees
	# tlen1, tlen2 == original tree size
	# deli1, deli2 == deleted intraspecific duplications
	#Initialize Values
	biol_distance = 0
	strict_distance = 0
	num_subtree1=len(col1)
	num_subtree2=len(col2)
	info = {}
	#We will always start with the tree with the largest number of subtrees
	if num_subtree1 >= num_subtree2:
		initial_collection = col1
		final_collection = col2
		tree_length1 = tlen1
		tree_length2 = tlen2
		deleted1 = deli1
		deleted2 = deli2
	else:
		initial_collection = col2
		final_collection = col1
		tree_length2 = tlen1
		tree_length1 = tlen2
		deleted1 = deli2
		deleted2 = deli1
	#Memorize data
	for ident1,nw1 in initial_collection.iteritems():
		info[ident1] = {}
		for ident2,nw2 in final_collection.iteritems():
			lentra,lentrb,min_dist,rf,term=get_minimum_data(nw2,nw1,deleted1,deleted2,tree_mode,species_position)
			info[ident1][ident2] = {}
			info[ident1][ident2]["sizeA"] = 1.0 - float(lentra)/float(tree_length1)
			info[ident1][ident2]["sizeB"] = 1.0 - float(lentrb)/float(tree_length2)
			info[ident1][ident2]["lenA"] = lentra
			info[ident1][ident2]["lenB"] = lentrb
			info[ident1][ident2]["distance"] = min_dist
			info[ident1][ident2]["RF"] = rf
			info[ident1][ident2]["first_term"] = term
	biol_distance = calc_biol_distance(info,tree_length1,tree_length2)
	strict_distance = calc_strict_distance(initial_collection,final_collection,info,tree_length1,tree_length2,tree_mode,species_position)
	return strict_distance,biol_distance

#Rewrites the information into string so that it is easier to filter them
def convert_distances(col1,col2,info):
	all_info = []
	colection1 = []
	colection2 = []
	added_keys = []
	for key1,values in info.iteritems():
		list_info = []
		colection1.append(col1[key1])
		for key2,information in values.iteritems():
			if key2 not in added_keys:
				added_keys.append(key2)
				colection2.append(col2[key2])
			string = []
			dist = "%.6f" % information["distance"]
			mida = information["sizeA"] + information["sizeB"]
			string = dist + "_" + str(information["RF"]) + "_" + str(mida) + "_" + str(information["lenA"]) + "_" + str(information["lenB"])
#			string = dist + "_" + str(information["RF"]) + "_" + str(mida)
			list_info.append(string)
		all_info.append(list_info)
	return colection1,colection2,all_info

#This function unites all the distance values in one single distance.
def calc_strict_distance (col1,col2,info,tlen1,tlen2,tree_mode,species_position):
	#Initialize values
	all_info,colection1,colection2, final = [],[],[], []
	tlen1, tlen2 = float(tlen1),float(tlen2)
	term1,term2, div1, div2, term_add, toadd = 0, 0, 0, 0, 0.0, 0
	#Create strings
	colection1,colection2,all_info = convert_distances(col1,col2,info)
	#Select lowest strings
	while colection1 != [] and colection2 != []:
		columns = [ getMinAndIndex(sub) for sub in all_info ]
		value,row = getMinAndIndex( [s[0] for s in columns ] )
		final.append(value)
		del all_info[row]
		del colection1[row]
		del colection2[columns[row][1]]
		for llista in all_info:
			del llista[columns[row][1]]
	#We add for the subtrees where we found no match => subtree_len / total_tree_len
	if colection1 == []:	
		toadd = "2"
		for nw in colection2:
			if type(nw) is str:
				tree = sr.load_tree(nw,tree_mode,species_position,0)
				br = len(tree.get_leaf_names())
			else:
				br = len(nw)
			term_add = term_add + float(br)/tlen2
	elif colection2 == []:
		toadd = "1"
		for nw in colection1:
			if type(nw) is str:
				tree = sr.load_tree(nw,tree_mode,species_position,0)
				br = len(tree.get_leaf_names())
			else:
				br = len(nw)
			term_add = term_add + float(br)/tlen1

	for a in final:
		info = a.split("_")
		dist=float(info[0])
		brA=float(info[3])
		brB=float(info[4])
		tlen1=float(tlen1)
		tlen2=float(tlen2)
		term1 = term1 + (dist*(brA/tlen1))
		term2 = term2 + (dist*(brB/tlen2))
		div1 = div1 + (brA/tlen1)
		div2 = div2 + (brB/tlen2)
	
	if toadd == "1":
		term1 = term1 + term_add
		div1 = div1 + term_add
	elif toadd == "2":
		term2 = term2 + term_add
		div2 = div2 + term_add
	
	distance = 0.5*(term1/div1) + 0.5*(term2/div2)
	return distance

def calc_biol_distance(info,tlen1,tlen2):
	# We transform the distance list to a list that contains all the information
	info_by_row = {}
	info_by_column = {}
	columns = []
	rows = []
	tlen1 = float(tlen1)
	tlen2 = float(tlen2)
	taken_columns=[]
	chosen = []
	term1 = 0.0
	term2 = 0.0
	div1 = 0.0
	div2 = 0.0
	for key1,values in info.iteritems():
		list_info = []
		if key1 not in rows:
			rows.append(key1)
		if key1 not in info_by_row:
			info_by_row[key1] = {}
		for key2,information in values.iteritems():
			if key2 not in columns:
				columns.append(key2)
			if key2 not in info_by_column:
				info_by_column[key2] = {}
			string = []
			mida = information["sizeA"] + information["sizeB"]
			mida = "%.3f" %mida
			ft = "%.6f" %information["first_term"]
			string = ft + "_" + str(information["RF"]) + "_" + str(mida) + "_" + str(information["lenA"]) + "_" + str(information["lenB"])
			info_by_row[key1][key2] = string
			info_by_column[key2][key1] = string
	
	#We select for each tree its best match
	for row in rows:
		minim = ""
		for col in columns:
			if minim == "" or info_by_row[row][col] <= minim:
				to_take = ""
				provisional = ""
				if info_by_row[row][col] == minim:
					if col in taken_columns:
						provisional = col
					else:
						to_take = col
				else:
					to_take = col
					minim = info_by_row[row][col]
		if to_take:
			taken_columns.append(to_take)
			chosen.append(info_by_row[row][to_take])
		else:
			chosen.append(info_by_row[row][provisional])
	#For the trees in col1
	for col in columns:
		if col not in taken_columns:
			minim = ""
			for row in rows:
				if minim == "" or info_by_column[col][row] <= minim:
					minim = info_by_column[col][row]
			chosen.append(minim)
	
	tlen1=float(tlen1)
	tlen2=float(tlen2)
	#We calculate the speciation distance for the pairs of trees we've picked
	for c in chosen:
		brA = float(c.split("_")[3])
		brB = float(c.split("_")[4])
		dist = float(c.split("_")[0])
		term1 = term1 + (dist*(brA/tlen1))
		term2 = term2 + (dist*(brB/tlen2))
		div1 = div1 + (brA/tlen1)
		div2 = div2 + (brB/tlen2)
	biol_dist = 0.5*(term1/div1) + 0.5*(term2/div2)
	return biol_dist



# This function is the main program used to calculate the comparison data between two trees. It returns minim distance, number of original branches in both trees, the robinson foulds, the winner newick and the result of rf/rf_max

def get_minimum_data (nw2,nw1,deli1,deli2,tree_mode,species_position):
	min_dist="none"
	branchA=0
	branchB=0
	rf= ""
	rf_k = ""
	rf,dist,lentra,lentrb,fterm = comparison(nw1,nw2,deli1,deli2,tree_mode,species_position)
	if dist < min_dist or min_dist == "none":
		rf_k=rf
		min_dist = dist
		nw = nw2
		branchA = lentra
		branchB = lentrb
		term = fterm
	elif dist == min_dist:
		if lentra>branchA or lentrb>branchB or rf<rf_k:
			if rf < rf_k:
				rf_k=rf
				min_dist = dist
				nw = nw2
				branchA = lentra
				branchB = lentrb
				term = fterm

	return branchA,branchB,min_dist,rf_k,term

######################################################################################################################
# Comparison functions
######################################################################################################################

# This function decides if we have two trees, two loners or two of each and calculates their distance acordingly
def comparison(nw1,nw2,deli1,deli2,tree_mode,species_position):
	#This function calculates the distance between two objects
	if not type(nw1) is list and not type(nw2) is list:
		rf,dist,lenA,lenB,fterm = compare_two_trees(nw1,nw2,deli1,deli2,tree_mode,species_position)
	elif type(nw1) is list and type(nw2) is list:
		rf,dist,lenA,lenB,fterm = compare_two_loners(nw1,nw2,deli1,deli2,tree_mode,species_position)
	else:
		rf,dist,lenA,lenB,fterm = compare_tree_and_loner(nw1,nw2,deli1,deli2,tree_mode,species_position)
		
	return rf,dist,lenA,lenB,fterm
	
def compare_two_trees(nw1,nw2,deli1,deli2,tree_mode,species_position):
	#If the objects are two trees
	tra,trb,go,prune,totalbr,lenA,lenB=prune_trees(nw1,nw2,tree_mode,species_position)
	lenA = check_removed_intraspecific_tree(tra,nw1,lenA,deli1,tree_mode,species_position)
	lenB = check_removed_intraspecific_tree(trb,nw2,lenB,deli2,tree_mode,species_position)
	if go == "yes":
		rf,rf_max = calc_rf(tra,trb,tree_mode,species_position)
		dist,fterm=calc_indiv_distance(rf,prune,totalbr,rf_max)
	elif go == "no":
		rf = 1
		rf_max = 1
		dist,fterm = calc_indiv_distance(rf,prune,totalbr,rf_max)
	elif go == "no_but":
		rf = 0
		rf_max = 1
		dist,fterm = calc_indiv_distance(rf,prune,totalbr,rf_max)
	return rf,dist,lenA,lenB,fterm
	
def compare_two_loners(nw1,nw2,deli1,deli2,tree_mode,species_position):
	toadd1 = check_removed_intraspecific_loner(nw1,deli1)
	toadd2 = check_removed_intraspecific_loner(nw2,deli2)
	#If the objects are two loners:
	nw1 = remake_list(nw1,tree_mode,species_position)
	nw2 = remake_list(nw2,tree_mode,species_position)
	lon1 = set(nw1)
	lon2 = set(nw2)
	intersec = lon1.intersection(lon2)
	if len(intersec) == 0:
		dist = 1
		rf = 1
		fterm = 1
	else :
		rf = 0
		rf_max = 1
		pr = len(lon1) + len(lon2) - 2*len(intersec)
		tr = len(lon1) + len(lon2)
		dist,fterm = calc_indiv_distance(rf,pr,tr,rf_max)
	lenA=len(lon1) + toadd1
	lenB=len(lon2) + toadd2
	return rf,dist,lenA,lenB,fterm
	
def compare_tree_and_loner(nw1,nw2,deli1,deli2,tree_mode,species_position):
	#if we have a tree and a loner
	if type(nw1) is list:
		tree = sr.load_tree(nw2,tree_mode,species_position,0)
		lon = nw1
		nw = nw2
		delitree = deli2
		deliloner = deli1
		order = 1
	else :
		tree = sr.load_tree(nw1,tree_mode,species_position,0)
		lon = nw2
		nw = nw1
		delitree = deli1
		deliloner = deli2
		order = 2
	tree = change_leaves(tree)
	lon = remake_list(lon,tree_mode,species_position)
	lon2 = set(tree.get_leaf_names())
	lon1 = set(lon)
	intersec = lon2.intersection(lon1)
	if len(intersec) == 0:
		dist = 1
		rf = 1
		fterm = 1
	else:
		rf = 0
		rf_max = 1
		pr = len(lon2) + len(lon1) - 2*len(intersec)
		tr = len(lon2) + len(lon1)
		dist,fterm = calc_indiv_distance(rf,pr,tr,rf_max)
	lenA = len(lon1)
	toadd = check_removed_intraspecific_loner(lon,deliloner)
	lenloner = lenA + toadd
	lenB = len(lon2)
	lentree = check_removed_intraspecific_tree(tree,nw,lenB,delitree,tree_mode,species_position)
	if order == 1:
		lenB = lentree
		lenA = lenloner
	elif order == 2:
		lenA = lentree
		lenB = lenloner
	return rf,dist,lenA,lenB,fterm

#This functions modify the number of species in each subtree to suit the fact that there were intraspecific duplications deleted
def check_removed_intraspecific_tree (tree,nw,len,deleted,tree_mode,species_position):
	if not deleted == {} and tree and nw:
		if type(tree) is str:
			tree = sr.load_tree(tree,tree_mode,species_position,0)
		if type(nw) is str:
			nw = sr.load_tree(nw,tree_mode,species_position,0)
		toadd = 0
		for leaf in nw.get_leaves():
			if leaf.name in deleted and leaf.species in tree.get_leaf_names():
				code = set(deleted[leaf.name])
				for i in code:
					toadd = toadd + 1
		toadd = len + toadd
	else :
		toadd = len
		
	return toadd

def check_removed_intraspecific_loner(nw,deleted):
	toadd = 0
	for leaf in nw:
		if leaf in deleted:
			code = set(deleted[leaf])
			for i in code:
				toadd = toadd + 1
	return toadd


# Functions used to change the name of the leafs to their species codes
# This function returns the species code for a given leaf name
def get_species_name(node_name_string,tree_mode,species_position):
	if tree_mode == "s":
		spe = node_name_string[0:3]
	else:
		num = tree_mode.replace("s","")
		if num == "all":
			spe = node_name_string
		elif num == "phylomedb":
			spe = node_name_string.split("_")[1]
		else:
			if species_position == "1":
				if num == "_":
					spe = node_name_string.split("_")[0]
				elif num == "-":
					spe = node_name_string.split("-")[0]
				elif num == "#":
					spe = node_name_string.split("#")[0]
				elif num == ".":
					spe = node_name_string.split(".")[0]
				else:
					spe = node_name_string[0:int(num)]
			elif species_position == "-1":
				if num == "_":
					spe = node_name_string.split("_")[-1]
				elif num == "-":
					spe = node_name_string.split("-")[-1]
				elif num == "#":
					spe = node_name_string.split("#")[-1]
				elif num == ".":
					spe = node_name_string.split(".")[-1]
				else:
					num = len(node_name_string) - int(num)
					spe = node_name_string[num:]
	return spe


#This function changes the codes of a list to their species codes so that two trees can be compared, this is used for loners, who do not have a tree
def remake_list(list,tree_mode,species_position):
	list1 = []
	for i in list:
		name = get_species_name(i,tree_mode,species_position)
		list1.append(name)
	return list1
	
#This function is used to read a newick and change its leaves to their species names.
def process_tree (nw,tree_mode,species_position):
	if type(nw) is str:	
		tree = sr.load_tree(nw,tree_mode,species_position,0)
	tree = change_leaves(tree)
	return tree	

#These functions are used to obtain the pruned trees of two trees.
def prune_trees(nw1,nw2,tree_mode,species_position):
	go = "no"
	pr = 0
	rf_max = 0
	pruned_tree1 = ""
	pruned_tree2 = ""
	
	#Turn nw to trees and change leaf names to their species name
	tree1 = process_tree(nw1,tree_mode,species_position)
	tree2 = process_tree(nw2,tree_mode,species_position)
	
	#List species in tree
	list1=set(tree1.get_leaf_names())
	list2=set(tree2.get_leaf_names())
	
	#Get the common species in both trees
	common_leaves=list1.intersection(list2)
		
	#Tree lengths:
	length_tree1 = len(list1)
	length_tree2 = len(list2)
	total_length = length_tree1 + length_tree2
	num_pruned_leaves_tree1 = len(list1) - len(common_leaves)
	num_pruned_leaves_tree2 = len(list2) - len(common_leaves)
	
	#robinson_foulds_maximum_distance
	rf_max=int(len(common_leaves))*2-4
	
	#Prune trees (only if there are at least 3 species in common)
	if len(common_leaves)>=3:
		pruned_tree1=prune_trees1(tree1,common_leaves)
		pruned_tree2=prune_trees1(tree2,common_leaves)
		pruned_leaves = num_pruned_leaves_tree1 + num_pruned_leaves_tree2
		go="yes"
	elif len(common_leaves) == 0:
		go="no"
		pruned_leaves = length_tree1 + length_tree2
	else :
		go="no_but"
		pruned_leaves = length_tree1 + length_tree2 - 2*len(common_leaves)
	return pruned_tree1,pruned_tree2,go,pruned_leaves,total_length,length_tree1,length_tree2

def prune_trees1(tree,leaves_to_keep):
	for leaf in tree.get_leaves():
		if not leaf.species in leaves_to_keep:
			l=tree.get_leaves_by_name(leaf.name)[0]
			lup=l.up
			lup.remove_child(l)
			lup.delete()
	tree = check_root(tree)
	newick=""
	if not len(tree.get_children())<2:
		newick=add_distances(tree)
		newick=tree.write()
	return newick

## Functions used to calculate different parts of the program

#This function calculates the distance between two trees taking into account the pruned leaves between them
def calc_indiv_distance(rf,branques_tretes,branques_a_b,rf_max):
	rf=float(rf)
	prune=float(branques_tretes)
	totalbr=float(branques_a_b)
	if rf_max == 0:
		rf_max = 1.00
	else:
		rf_max=float(rf_max)
	dist = (rf/rf_max*(totalbr - prune) + (prune))/totalbr
	#dist=0.5*(rf/rf_max)+0.5*(prune/totalbr)
	fterm = rf/rf_max
	return dist,fterm
	
#This functions calculates the robinson foulds distance
	
def calc_rf(nw1,nw2,tree_mode,species_position):
	r1,left,joint = get_partitions(nw1,tree_mode,species_position)
	r2,left,joint = get_partitions(nw2,tree_mode,species_position)
	inters = r1.intersection(r2)
	if len(r1) == len(r2):
		rf = (len(r1) - len(inters)) * 2
	else :
		rf = (len(r1) - len(inters)) + (len(r2) - len(inters))
	rf_max = len(r1) + len(r2)
	return rf,rf_max

def get_partitions(nw,tree_mode,species_position):
	tree = sr.load_tree(nw,tree_mode,species_position,0)
	orig_proteins = set(tree.get_leaf_names())
	left = []
	right = []
	joint = []
	for node in tree.get_descendants():
		if not node.is_leaf():
			list1 = set(node.get_leaf_names())
			if node.get_sisters():
				list2 = set((node.get_sisters()[0]).get_leaf_names())
				ls1 = sorted(list(list1))
				ls2 = sorted(list(list2))
				r = '|'.join(ls1)
				l = '|'.join(ls2)
				j = r + "-" + l
				right.append(r)
				left.append(l)
				joint.append(j)
	right = set(right)
	left = set(left)
	joint = set(joint)
	return right,left,joint

######################################################################################################
# Print functions
######################################################################################################

def print_results_main(compared,print_info,file1,file2,lc1,lc2,strict_distance,biol_distance,outfile,colection1,colection2,toomany):
	if compared:
		if print_info["oneline"]:
			print_oneline(file1,file2,lc1,lc2,compared,strict_distance,biol_distance,outfile,colection1,colection2,print_info)
		else:
			print_results(file1,file2,lc1,lc2,compared,strict_distance,biol_distance,outfile,colection1,colection2,print_info)
	else:
		if toomany :
			print_toomany(file1,file2,lc1,lc2,outfile)
		else:
			print_toofew(file1,file2,outfile)
	
def print_oneline(file1,file2,lc1,lc2,compared,strict_distance,biol_distance,outfile,colection1,colection2,print_info):
	if compared:
		print >>outfile,"\tTree 1\tTree 2\tStrict distance\tEvolutionary distance"
		print >>outfile,"----------------------------------------------------------------------------"
		print >>outfile,"Results\t"+file1+"\t"+file2+"\t%.3f\t%.3f" % (strict_distance,biol_distance)
	else:
		print >>outfile,file1+" and "+file2+" were not compared"

def print_results(file1,file2,lc1,lc2,compared,strict_distance,biol_distance,outfile,colection1,colection2,print_info):
	if print_info["verbose"]:
		print_header(outfile,file1,file2,lc1,lc2)

	if compared:
		if print_info["pall"]:
			print_strict(outfile,strict_distance,print_info["verbose"])
			print_biological(outfile,biol_distance,print_info["verbose"])
			print_subtrees(outfile,file1,file2,colection1,colection2)
		else:
			if print_info["pstr"]:
				print_strict(outfile,strict_distance,print_info["verbose"])
			if print_info["pbiol"]:
				print_biological(outfile,biol_distance,print_info["verbose"])
			if print_info["psub"]:
				print_subtrees(outfile,file1,file2,colection1,colection2)
		print >>outfile,"\n"
	else:
		print >>outfile,"These two trees were not compared."
		print >>outfile,"\n"

def print_toomany (file1,file2,lc1,lc2,outfile):
	print >>outfile,"\n"
	print >>outfile,"There were too many pruned trees for the limits you have used"
	print >>outfile,"\n"


def print_toofew (file1,file2,outfile):
	print >>outfile,"\n"
	print >>outfile,"There were not enough pruned trees to compare these two trees"

def print_configuration(default,general_info,info_tree1,info_tree2,print_info,outfile):
	if default:
		print >>outfile,"No valid configuration file provided. Running on default values:"
	else:
		print >>outfile,"This is your current configuration:"
	print >>outfile,"\n**General Information**\n"
	if general_info["orto_mode"] == "s":
		print >>outfile,"Orthology prediction = Species overlap"
	elif general_info["orto_mode"] == "r":
		print >>outfile,"Orthology prediction = Tree reconciliation"
		print >>outfile,"Species tree = "+general_info["species_tree"]
	elif general_info["orto_mode"] == "m":
		print >>outfile,"Orthology prediction = User defined"
	if general_info["overlap"]:
		print >>outfile,"Punned tree overlap = Yes"
	else:
		print >>outfile,"Prunned tree overlap = No"
	if general_info["limit"] == 0:
		print >>outfile,"Limits = None"
	else:
		print >>outfile,"Limits = "+str(general_info["limit"])
	
	
	print >>outfile,"\n**Information for Tree 1**\n"
	if info_tree1["spec_filter"] != None:
		print >>outfile,"Species filter = "+info_tree1["spec_filter"]
	else:
		print >>outfile,"Species filter = None"
	if info_tree1["prot_filter"] != None:
		print >>outfile,"Protein filter = "+info_tree1["prot_filter"]
	else:
		print >>outfile,"Protein filter = None"
	if info_tree1["kind_root"] == "m":
		print >>outfile,"Root = Midpoint"
	elif info_tree1["kind_root"] == "f":
		print >>outfile,"Root = Farthest species"
	elif info_tree1["kind_root"] == "fi":
		print >>outfile,"Root = Farthest species improved"
	elif info_tree1["kind_root"] == "s":
		print >>outfile,"Root = Specific protein or species"
	elif info_tree1["kind_root"] == "n":
		print >>outfile,"Root = Already rooted"
	elif info_tree1["kind_root"] == "d":
		print >>outfile,"Root = Minimize duplication nodes"
	if info_tree1["bootstrap"] == None:
		print >>outfile,"Support filtering = None"
	else:
		print >>outfile,"Support filtering = "+str(info_tree1["bootstrap"])
		
	print >>outfile,"\n**Information for Tree 2**\n"
	if info_tree2["spec_filter"] != None:
		print >>outfile,"Species filter = "+info_tree2["spec_filter"]
	else:
		print >>outfile,"Species filter = None"
	if info_tree2["prot_filter"] != None:
		print >>outfile,"Protein filter = "+info_tree2["prot_filter"]
	else:
		print >>outfile,"Protein filter = None"
	if info_tree2["kind_root"] == "m":
		print >>outfile,"Root = Midpoint"
	elif info_tree2["kind_root"] == "f":
		print >>outfile,"Root = Farthest species"
	elif info_tree2["kind_root"] == "fi":
		print >>outfile,"Root = Farthest species improved"
	elif info_tree2["kind_root"] == "s":
		print >>outfile,"Root = Specific protein or species"
	elif info_tree2["kind_root"] == "n":
		print >>outfile,"Root = Already rooted"
	elif info_tree2["kind_root"] == "d":
		print >>outfile,"Root = Minimize duplication nodes"
	if info_tree2["bootstrap"] == None:
		print >>outfile,"Support filtering = None"
	else:
		print >>outfile,"Support filtering = "+str(info_tree2["bootstrap"])
	
	
	print >>outfile, "\n**Printing information**\n"
	if print_info["pall"]:
		print >>outfile,"Print all = Yes"
	else:
		if print_info["psub"]:
			print >>outfile,"Print pruned trees = Yes"
		else:
			print >>outfile,"Print pruned trees = No"
		if print_info["pbiol"]:
			print >>outfile,"Print speciation distance = Yes"
		else:
			print >>outfile,"Print speciation distance = No"
		if print_info["pstr"]:
			print >>outfile,"Print strict distance = Yes"
		else:
			print >>outfile,"Print strict distance = No"
		if print_info["ptime"]:
			print >>outfile,"Print time = Yes"
		else:
			print >>outfile,"Print time = No"
		if print_info["verbose"]:
			print >>outfile,"Print verbose = Yes"
		else:
			print >>outfile,"Print verbose = No"

def print_subtrees(outfile,file1,file2,colection1,colection2):
	print >>outfile,str(len(colection1))+" trees for tree A:",file1
	for col in colection1:
		print >>outfile,col
	if file2 != None and colection2 != None:
		print >>outfile,"\n"+str(len(colection2))+" trees for tree B:",file2
		for col in colection2:
			print >>outfile,col
	print >>outfile,"\n***\n"

def print_header(outfile,file1,file2,lc1,lc2):
	print >>outfile,"\nResults for:\n\n*****************************************************************"
	print >>outfile,"First tree file: "+file1+", number of pruned trees: "+str(lc1)
	print >>outfile,"Second tree file "+file2+", number of pruned trees: "+str(lc2)
	print >>outfile,"*****************************************************************"
	print >>outfile,"\n"
	
def print_strict(outfile,strict_distance,verbose):
	if verbose:
		print >>outfile,"The strict distance between the two trees is: %.3f" % strict_distance
		print >>outfile,"\n***\n"
	else:
		print >>outfile, "%.3f" % strict_distance

def print_biological(outfile,biol_distance,verbose):
	if verbose:
		print >>outfile,"The speciation distance between the two trees is: %.3f" % biol_distance
		print >>outfile,"\n***\n"
	else:
		print >>outfile, "%.3f" % biol_distance

def check_input_files(file1,file2):
	is_rooted1,is_rooted2 = False, False
	species_check1,species_check2 = True,True
	if not os.path.exists(file1) or not os.path.exists(file2):
		print "TreeKo was unable to find one or both of the tree files, please check your command line"
		exit()
	else:
		nw1 = read_tree_file(file1)
		nw2 = read_tree_file(file2)
		try:
			tree = ete2.PhyloTree(nw1)
			if len(tree.get_children()) == 2:
				is_rooted1 = True
			if len(tree.get_leaf_names()) != len(set(tree.get_leaf_names())):
				species_check1 = False
				
		except:
			print "The file provided as tree 1 does not have a valid format"
			exit()
		try:
			tree = ete2.PhyloTree(file2)
			if len(tree.get_children()) == 2:
				is_rooted2 = True
			if len(tree.get_leaf_names()) != len(set(tree.get_leaf_names())):
				species_check2 = False

		except:
			print "The file provided as tree 2 does not have a valid format"
			exit()
	return nw1,nw2,is_rooted1,is_rooted2,species_check1,species_check2

def parse_configuration_file(options_conf_file,outfile,is_rooted1,is_rooted2,seed,pconfig):
	if not options_conf_file or not os.path.exists(options_conf_file):
		
		#Load initial parameters
		print_info = load_initial_data("p")
		general_info = load_initial_data("g")
		info_tree1 = load_initial_data("t")
		info_tree2 = load_initial_data("t")
		if seed != None:
			info_tree2["prot_filter"] = seed
			info_tree1["prot_filter"] = seed
		default = True
		
		#Modify needed info:
		
		print_info["pall"] = True
		print_info["verbose"] = True
		if is_rooted1:
			info_tree1["kind_root"] = "n"
		if is_rooted2:
			info_tree2["kind_root"] = "n"
		if pconfig == True:
			print_configuration(default,general_info,info_tree1,info_tree2,print_info,outfile)
	else:
		general_info,info_tree1,info_tree2,print_info = parse_conf_file(options_conf_file,is_rooted1,is_rooted2,seed)
		default = False
		if print_info["pconfig"]:
			print_configuration(default,general_info,info_tree1,info_tree2,print_info,outfile)
	return default,general_info,info_tree1,info_tree2,print_info

def read_tree_file(file):
	i = 0
	nw = ""
	for line in open(file):
		line = line.strip()
		if i == 0:
			if ";" not in line:
				nw = nw+line
			else:
				nw = nw+line
				i = 1
	return nw

###################################################################################################
# Main program
###################################################################################################

def tree_comparison(file1,file2,conf_file,outfile,seed):

	#Initialize variables
	now=datetime.datetime.now()
	colection1,colection2,distances,brlengthA,brlengthB=[],[],[],[],[]
	compared,distance,print_resu,launch_treeKo = True,False,False,True
	#Check input/output files
	nw1,nw2,is_rooted1,is_rooted2,check_species1,check_species2 = check_input_files(file1,file2)
	
	
	#Parse configuration file
	default,general_info,info_tree1,info_tree2,print_info = parse_configuration_file(conf_file,outfile,is_rooted1,is_rooted2,seed,True)

	#If needed, check tree overlap
	if general_info["overlap"] > 0:
		launch_treeKo = check_overlap(nw1,nw2,general_info["tree_mode"],general_info["species_position"])
		if not launch_treeKo :
			print "No overlap exists between "+file1+ " and " +file2
			sys.exit()

	#For reconciliation, read species tree
	if general_info["orto_mode"] == "r":
		try:
			general_info["species_tree"] = ete2.PhyloTree(general_info["species_tree"])
		except:
			sys.exit("Unable to load the species tree provided, please check your species tree file")
		if len(general_info["species_tree"].get_children()) == 3:
			sys.exit("Species tree unrooted, please root your species tree before continuing")
	
	
	#Get subtrees
	colection1,total_tree_length1,deli1,stop1=sr.make_subtrees(nw1,general_info,info_tree1,check_species1)
	#colection1 = filter_colection(colection1,general_info,info_tree1)

	if stop1 == "NOTROOTED":
		sys.exit("Unable to root tree1, please chose another tree rooting program")
	elif stop1 == "ONESPECIES":
		sys.exit("Tree 1 only contains 1 species, unable to compare this tree")
	colection2,total_tree_length2,deli2,stop2=sr.make_subtrees(nw2,general_info,info_tree2,check_species2)
	#colection2 = filter_colection(colection2,general_info,info_tree2)

	if stop2 == "NOTROOTED":
		sys.exit("Unable to root tree2, please chose another tree rooting program")
	elif stop2 == "ONESPECIES":
		sys.exit("Tree 2 only contains 1 species, unable to compare this tree")
		
	if not print_info["pbiol"] and not print_info["pstr"] and not print_info["pall"] and not print_info["oneline"]:
		launch_treeKo = False
		if print_info["psub"]:
			if not stop1 and not stop2:
				print_subtrees(outfile,file1,file2,colection1,colection2)
			else:
				print_toomany(file1,file2,len(colection1),len(colection2),outfile)
	if stop1 or stop2:
		launch_treeKo = False
		print_toomany(file1,file2,len(colection1),len(colection2),outfile)
	#Start comparison
	if launch_treeKo:
		cole1 = {}
		cole2 = {}
		i = 0
		for col in colection1:
			code = "tree1_"+str(i)
			i = i + 1
			cole1[code] = col
		
		i = 0
		for col in colection2:
			code = "tree2_"+str(i)
			i = i + 1
			cole2[code] = col

		lc1 = len(colection1)
		lc2 = len(colection2)
		#Calulate distances
		toomany = False
		if (lc1 == 0 or lc2 == 0):
			compared=False
			toomany=False
		else:
			if (lc1 < general_info["limit"] and lc2 < general_info["limit"]) or general_info["limit"] == 0:
				strict_distance,biol_distance = compare_trees(cole1,cole2,total_tree_length1,total_tree_length2,deli1,deli2,general_info["tree_mode"],general_info["species_position"])
			else:
				compared=False
				toomany=True

		print_results_main(compared,print_info,file1,file2,lc1,lc2,strict_distance,biol_distance,outfile,colection1,colection2,toomany)
	if print_info["ptime"] or print_info["pall"]:
		calc_time(now,outfile)
	return biol_distance
